#include "main.h"
#include "create_automata.h"

#define Delta "2"
#define Gamma "1"

int N=3;

std::vector<automaton> get_lhs_automaton_concrete(msat_env env)
{

  std::vector<automaton> res_v;
  
  std::string init = "init";

  std::string t = "t", t0 = "t0", t1 = "t1";  
  std::string last = "last", last0 = "last0", last1 = "last1";

  std::string c = "c", c0 = "c0", c1 = "c1";
  std::string w = "w", w0 = "w0", w1 = "w1";
  
  for (int i = 0; i < N; i ++) {
    automaton station;

    std::string trying = "trying" + std::to_string(i);
    std::string set = "set" + std::to_string(i);
    std::string retrying = "retrying" + std::to_string(i);
    std::string enter = "enter" + std::to_string(i);
    std::string exit = "exit" + std::to_string(i);

    station.alphabet.insert(init);
    station.alphabet.insert(trying);
    station.alphabet.insert(set);
    station.alphabet.insert(retrying);
    station.alphabet.insert(enter);
    station.alphabet.insert(exit);
    
    station.vars.insert(t);
    station.vars.insert(last);

    station.vars.insert(c);
    station.vars.insert(w);

    std::string x = "var" + std::to_string(i) + "x";
    std::string x0 = "var" + std::to_string(i) + "x0";
    std::string x1 = "var" + std::to_string(i) + "x1";

    station.vars.insert(x);

    DWA_VARIABLE_TYPE=msat_get_rational_type(env); // global type redefinition
    create_msat_variables(env,station.vars,DWA_VARIABLE_TYPE);

    int s0=5*i, s1=5*i+1, s2=5*i+2, s3=5*i+3, s4=5*i+4;

    station.init_states.insert(s0);
    station.final_states.insert(s1);

    create_rule(station.trans,env,s0,init,s1,"(and (= " + t1 + " 0) (= " + c1 + " 0) (= " + w1 + " 0) (= " + last1 + " (- 1)))");
    create_rule(station.trans,env,s1,trying,s2,"(and (<= " + t0 + " " + t1 + ") (= " + c0 + " " + c1 + ") (= " + w0 + " " + w1 + ") (= " + last0 + " (- 1)) (= " + last0 + " " + last1 + ") (= " + x1 + " " + t0 + ") (<= (- " + t1 + " " + x1 + ") " + Delta + "))");
    create_rule(station.trans,env,s2,set,s3,"(and (<= " + t0 + " " + t1 + ") (= " + c0 + " "+ c1 + ") (= (+ " + w0 + " 1) " + w1 + ") (= " + last1 + " " + std::to_string(i) + ") (= " + x1 + " " + t0 +") (<= (- " + t0 + " " + x0 + ") " + Delta + "))");
    create_rule(station.trans,env,s3,retrying,s1,"(and (<= " + t0 + " " + t1 + ") (= " + c0 + " "+ c1 + ") (= (- " + w0 + " 1) " + w1 + ") (distinct " + last0 + " " + std::to_string(i) + ") (= " + last0 + " " + last1 + ") (> (- " + t0 + "  " + x0 + ") " + Gamma + ") (= " + x1 + " " + x0 +"))");
    create_rule(station.trans,env,s3,enter,s4,"(and (<= " + t0 + " " + t1 + ") (= (+ " + c0 + " 1) " + c1 + ") (= (- " + w0 + " 1) " + w1 + ") (= " + last0 + " " + std::to_string(i) + ") (= " + last0 + " " + last1 + ") (= " + x1 + " " + x0 + ") (> (- " + t0 + " " + x0 + ") " + Gamma + "))");
    create_rule(station.trans,env,s4,exit,s1,"(and (<= " + t0 + " " + t1 + ") (= (- " + c0 + " 1) " + c1 + ") (= " + w0 + " " + w1 + ") (= " + last1 + " (- 1)) (= " + x1 + " " + x0 + "))");

    res_v.push_back(station);
  }

  return res_v;
}

automaton mutex_rhs_automaton(msat_env env) {
  automaton res;
  
  std::string init = "init";

  std::string t = "t", t0 = "t0", t1 = "t1";
  std::string last = "last", last0 = "last0", last1 = "last1";

  std::string c = "c", c0 = "c0", c1 = "c1";
  std::string w = "w", w0 = "w0", w1 = "w1";

  res.alphabet.insert(init);

  res.init_states.insert(0);
  res.final_states.insert(0);

  res.vars.insert(t);
  res.vars.insert(last);
  res.vars.insert(c); 
  res.vars.insert(w);

  create_rule(res.trans,env,0,init,0,"true");

  for (int i = 0; i < N; i ++) {
    std::string trying = "trying" + std::to_string(i);
    std::string set = "set" + std::to_string(i);
    std::string retrying = "retrying" + std::to_string(i);
    std::string enter = "enter" + std::to_string(i);
    std::string exit = "exit" + std::to_string(i);

    res.alphabet.insert(trying);
    res.alphabet.insert(set);
    res.alphabet.insert(retrying);
    res.alphabet.insert(enter);
    res.alphabet.insert(exit);

    create_rule(res.trans,env,0,trying,0,"(and (<= " + t0 + " " + t1 + ") (<= 0 " + c0 + ") (<= " + c0 + " 1))");
    create_rule(res.trans,env,0,set,0,"(and (<= " + t0 + " " + t1 + ") (<= 0 " + c0 + ") (<= " + c0 + " 1))");
    create_rule(res.trans,env,0,retrying,0,"(and (<= " + t0 + " " + t1 + ") (<= 0 " + c0 + ") (<= " + c0 + " 1))");
    create_rule(res.trans,env,0,enter,0,"(and (<= " + t0 + " " + t1 + ") (<= 0 " + c0 + ") (<= " + c0 + " 1))");
    create_rule(res.trans,env,0,exit,0,"(and (<= " + t0 + " " + t1 + ") (<= 0 " + c0 + ") (<= " + c0 + " 1))");
  }

  return res;
}

automaton live_rhs_automaton(msat_env env) {
  automaton res;

  std::string init = "init";

  std::string t = "t", t0 = "t0", t1 = "t1";
  std::string last = "last", last0 = "last0", last1 = "last1";

  std::string c = "c", c0 = "c0", c1 = "c1";
  std::string w = "w", w0 = "w0", w1 = "w1";

  res.alphabet.insert(init);

  res.init_states.insert(0);
  res.final_states.insert(1);

  res.vars.insert(t);
  res.vars.insert(last);
  res.vars.insert(c);
  res.vars.insert(w);

  create_rule(res.trans,env,0,init,1,"true");

  for (int i = 0; i < N; i ++) {
    std::string trying = "trying" + std::to_string(i);
    std::string set = "set" + std::to_string(i);
    std::string retrying = "retrying" + std::to_string(i);
    std::string enter = "enter" + std::to_string(i);
    std::string exit = "exit" + std::to_string(i);

    res.alphabet.insert(trying);
    res.alphabet.insert(set);
    res.alphabet.insert(retrying);
    res.alphabet.insert(enter);
    res.alphabet.insert(exit);

    create_rule(res.trans,env,1,trying,1,"(< " + w0 + " " + std::to_string(N) + ")");
    create_rule(res.trans,env,1,set,1,"(< " + w0 + " " + std::to_string(N) + ")");
    create_rule(res.trans,env,1,retrying,1,"(< " + w0 + " " + std::to_string(N) + ")");
    create_rule(res.trans,env,1,enter,1,"(< " + w0 + " " + std::to_string(N) + ")");
    create_rule(res.trans,env,1,exit,1,"(< " + w0 + " " + std::to_string(N) + ")");

    for (int j = 0; j < N; j ++) {

      create_rule(res.trans,env,j+1,trying,j+2,"true");
      create_rule(res.trans,env,j+1,set,j+2,"true");
      create_rule(res.trans,env,j+1,retrying,j+2,"true");
      create_rule(res.trans,env,j+1,enter,j+2,"true");
      create_rule(res.trans,env,j+1,exit,j+2,"true");

      /*
      create_rule(res.trans,env,j+1,trying,j+2,"(and (= " + w0 + " (- " + std::to_string(N) + " " + std::to_string(j) + " )) (= " + c0 + " 0))");
      create_rule(res.trans,env,j+1,set,j+2,"(and (= " + w0 + " (- " + std::to_string(N) + " " + std::to_string(j) + " )) (= " + c0 + " 0))");
      create_rule(res.trans,env,j+1,retrying,j+2,"(and (= " + w0 + " (- " + std::to_string(N) + " " + std::to_string(j) + " )) (= " + c0 + " 0))");
      create_rule(res.trans,env,j+1,enter,j+2,"(and (= " + w0 + " (- " + std::to_string(N) + " " + std::to_string(j) + " )) (= " + c0 + " 0))");
      create_rule(res.trans,env,j+1,exit,j+2,"(and (= " + w0 + " (- " + std::to_string(N) + " " + std::to_string(j) + " )) (= " + c0 + " 0))");
      */

      create_rule(res.trans,env,j+2,trying,1,"(= " + c0 + " 1)");
      create_rule(res.trans,env,j+2,set,1,"(= " + c0 + " 1)");
      create_rule(res.trans,env,j+2,retrying,1,"(= " + c0 + " 1)");
      create_rule(res.trans,env,j+2,enter,1,"(= " + c0 + " 1)");
      create_rule(res.trans,env,j+2,exit,1,"(= " + c0 + " 1)");
    }
  }

  return res;
}

automaton get_rhs_automaton_concrete(msat_env env) {
  return mutex_rhs_automaton(env);
}

  
