#include "main.h"
#include "create_automata.h"

#define L "1"
#define U "2"

#define P  "5"
#define ST "1"
#define SR "1"

#define P_plus_ST "6"
#define P_minus_ST "4"
#define P_plus_SR "6"
#define P_minus_SR "4"

int K=2;

std::string init_symbol = "init";
std::string tick_symbol = "tick";

// std::string t_symbol = "tt";
// std::string r_empty = "rr_empty", r_false = "rr_false", r_true = "rr_true";

std::string t = "t", t0 = "t0", t1 = "t1";

std::vector<automaton> get_lhs_automaton_concrete(msat_env env)
{
  std::vector<automaton> res_v;
  DWA_VARIABLE_TYPE=msat_get_rational_type(env); // global type redefinition

  
  /*
   * Transmitter 
   */

  automaton transmitter;
  
  transmitter.alphabet.insert(init_symbol);
  transmitter.alphabet.insert(tick_symbol);

  std::string c_t = "ct", c_t0 = c_t + "0", c_t1 = c_t + "1";
  std::string Xt_t  = "var0Xt", Xt_t0 = Xt_t + "0", Xt_t1 = Xt_t + "1";
  std::string Xf_t  = "var0Xf", Xf_t0 = Xf_t + "0", Xf_t1 = Xf_t + "1";

  transmitter.vars.insert(t);
  transmitter.vars.insert(c_t);
  transmitter.vars.insert(Xt_t);
  transmitter.vars.insert(Xf_t);

  create_msat_variables(env,transmitter.vars,DWA_VARIABLE_TYPE);
  
  int init_t = 0, s0_t = init_t + 1, s1_t = s0_t + 1, s2_t = s0_t + 2;

  transmitter.init_states.insert(init_t);
  transmitter.final_states.insert(s0_t);

  create_rule(transmitter.trans,env,init_t,init_symbol,s0_t, "(and (= " + Xf_t1 + " 0) (= " + Xt_t1 + " 0))");

  // send false 
  create_rule(transmitter.trans,env,s0_t,tick_symbol,s1_t, "(and (<= " + t0 + " " + t1 + ") (< (- " + t0 + " " + c_t0 + ") " + P_plus_ST + ") (>= (- " + t0+ " " + c_t0 + ") " + P_minus_ST + ") (= " + c_t1 + " " + t0 + ") (< (- " + t1 + " " + c_t1 + ") " + P_plus_ST + ") (= " + Xf_t1 + " 1) (= " + Xt_t1 + " 0))");

  // send empty
  create_rule(transmitter.trans,env,s1_t,tick_symbol,s0_t, "(and (<= " + t0 + " " + t1 + ") (< (- " + t0 + " " + c_t0 + ") " + P_plus_ST + ") (>= (- " + t0+ " " + c_t0 + ") " + P_minus_ST + ") (= " + c_t1 + " " + t0 + ") (< (- " + t1 + " " + c_t1 + ") " + P_plus_ST + ") (= " + Xf_t1 + " 0) (= " + Xt_t1 + " 0))");

  // send true
  create_rule(transmitter.trans,env,s0_t,tick_symbol,s2_t, "(and (<= " + t0 + " " + t1 + ") (< (- " + t0 + " " + c_t0 + ") " + P_plus_ST + ") (>= (- " + t0+ " " + c_t0 + ") " + P_minus_ST + ") (= " + c_t1 + " " + t0 + ") (< (- " + t1 + " " + c_t1 + ") " + P_plus_ST + ") (= " + Xt_t1 + " 1) (= " + Xf_t1 + " 0))");

  // send empty
  create_rule(transmitter.trans,env,s2_t,tick_symbol,s0_t, "(and (<= " + t0 + " " + t1 + ") (< (- " + t0 + " " + c_t0 + ") " + P_plus_ST + ") (>= (- " + t0+ " " + c_t0 + ") " + P_minus_ST + ") (= " + c_t1 + " " + t0 + ") (< (- " + t1 + " " + c_t1 + ") " + P_plus_ST + ") (= " + Xt_t1 + " 0) (= " + Xf_t1 + " 0))");

  res_v.push_back(transmitter);

  /*
   * Receiver 
   */

  automaton receiver;
  
  receiver.alphabet.insert(init_symbol);
  receiver.alphabet.insert(tick_symbol);

  std::string c_r = "cr", c_r0 = c_r + "0", c_r1 = c_r + "1";
  std::string Xt_r  = "var" + std::to_string(K) + "Xt", Xt_r0 = Xt_r + "0", Xt_r1 = Xt_r + "1";
  std::string Xf_r  = "var" + std::to_string(K) + "Xf", Xf_r0 = Xf_r + "0", Xf_r1 = Xf_r + "1";
  std::string ack_r = "var" + std::to_string(K+1) + "ack", ack_r0 = ack_r + "0", ack_r1 = ack_r + "1";

  receiver.vars.insert(t);
  receiver.vars.insert(c_r);
  receiver.vars.insert(Xt_r);
  receiver.vars.insert(Xf_r);
  receiver.vars.insert(ack_r);

  create_msat_variables(env,receiver.vars,DWA_VARIABLE_TYPE);

  int init_r = s2_t + 1, s0_r = init_r + 1;

  receiver.init_states.insert(init_r);
  receiver.final_states.insert(s0_r);

  create_rule(receiver.trans,env,init_r,init_symbol,s0_r, "(and (= " + Xf_r1 + " 0) (= " + Xt_r1 + " 0))");

  // receive empty
  create_rule(receiver.trans,env,s0_r,tick_symbol,s0_r, "(and (<= " + t0 + " " + t1 + ") (< (- " + t0 + " " + c_r0 + ") " + P_plus_SR + ") (>= (- " + t0+ " " + c_r0 + ") " + P_minus_SR + ") (= " + c_r1 + " " + t0 + ") (< (- " + t1 + " " + c_r1 + ") " + P_plus_SR + ") (= " + Xt_r0 + " 0) (= " + Xf_r0 + " 0) (= " + ack_r1 + " 1))");

  // receive false 
  create_rule(receiver.trans,env,s0_r,tick_symbol,s0_r, "(and (<= " + t0 + " " + t1 + ") (< (- " + t0 + " " + c_r0 + ") " + P_plus_SR + ") (>= (- " + t0+ " " + c_r0 + ") " + P_minus_SR + ") (= " + c_r1 + " " + t0 + ") (< (- " + t1 + " " + c_r1 + ") " + P_plus_SR + ") (= " + Xt_r0 + " 0) (= " + Xf_r0 + " 1) (= " + ack_r1 + " 0))");

  // receive true
  create_rule(receiver.trans,env,s0_r,tick_symbol,s0_r, "(and (<= " + t0 + " " + t1 + ") (< (- " + t0 + " " + c_r0 + ") " + P_plus_SR + ") (>= (- " + t0+ " " + c_r0 + ") " + P_minus_SR + ") (= " + c_r1 + " " + t0 + ") (< (- " + t1 + " " + c_r1 + ") " + P_plus_SR + ") (= " + Xt_r0 + " 1) (= " + Xf_r0 + " 0) (= " + ack_r1 + " 0))");

  res_v.push_back(receiver);

  for (int i = 1; i <= K; i ++) {
    
    /*
     * C True
     */

    automaton c_true;

    // std::string ct_symbol = "c_true" + std::to_string(i);

    c_true.alphabet.insert(init_symbol);
    c_true.alphabet.insert(tick_symbol);

    std::string c_t = "var" + std::to_string(i) + "ct", c_t0 = c_t + "0", c_t1 = c_t + "1";
    std::string Xt_prev = "var" + std::to_string(i-1) + "Xt", Xt_prev0 = Xt_prev + "0", Xt_prev1 = Xt_prev + "1";
    std::string Xt = "var" + std::to_string(i) + "Xt", Xt0 = Xt + "0", Xt1 = Xt + "1";
    std::string ack_t = "var" + std::to_string(i+1) + "ack", ack_t0 = ack_t + "0", ack_t1 = ack_t + "1";

    c_true.vars.insert(t);
    c_true.vars.insert(c_t);
    c_true.vars.insert(Xt_prev);
    c_true.vars.insert(Xt);
    c_true.vars.insert(ack_t);
    
    create_msat_variables(env,c_true.vars,DWA_VARIABLE_TYPE);

    int ct_init = 15 * i + s0_r + 1, ct0 = ct_init + 1, ct1 = ct0 + 2, ct2 = ct0 + 3, ct3 = ct0 + 4;

    c_true.init_states.insert(ct_init);
    c_true.final_states.insert(ct0);

    create_rule(c_true.trans,env,ct_init,init_symbol,ct0, "(= " + Xt1 + " 0)");

    create_rule(c_true.trans,env,ct0,tick_symbol,ct1, "(and (<= " + t0 + " " + t1 + ") (= " + c_t1 + " " + t0 + ") (< (- " + t1 + " " + c_t1 + ") " + U + ") (= " + Xt_prev0 + " 1) (= " + ack_t0 + " 1) (= " + Xt1 + " 0))");

    create_rule(c_true.trans,env,ct1,tick_symbol,ct0, "(and (<= " + t0 + " " + t1 + ") (< (- " + t0 + " " + c_t0 + ") " + U + ") (or (= " + Xt_prev0 + " 0) (= " + ack_t0 + " 0)) (= " + Xt1 + " 0))");

    create_rule(c_true.trans,env,ct1,tick_symbol,ct2, "(and (<= " + t0 + " " + t1 + ") (< (- " + t0 + " " + c_t0 + ") " + U + ") (>= " + "(- " + t0 + " " + c_t0 + ") " + L + ") (= "+ Xt_prev0 + " 1) (= " + ack_t0 +" 1) (= " + Xt1 + " 1))");

    create_rule(c_true.trans,env,ct2,tick_symbol,ct3, "(and (<= " + t0 + " " + t1 + ") (= " + c_t1 + " " + t0 + ") (< (- " + t1 + " " + c_t1 + ") " + U + ") (= " + Xt_prev0 + " 0) (= " + ack_t0 + " 0) (= " + Xt1 + " 1))");

    create_rule(c_true.trans,env,ct3,tick_symbol,ct2, "(and (<= " + t0 + " " + t1 + ") (< (- " + t0 + " " + c_t0 + ") " + U + ") (or (= " + Xt_prev0 + " 1) (= " + ack_t0 + " 1)) (= " + Xt1 + " 1))");

    create_rule(c_true.trans,env,ct3,tick_symbol,ct0, "(and (<= " + t0 + " " + t1 + ") (< (- " + t0 + " " + c_t0 + ") " + U + ") (>= " + "(- " + t0 + " " + c_t0 + ") " + L + ") (= "+ Xt_prev0 + " 0) (= " + ack_t0 + " 0) (= " + Xt1 + " 0))");    

    res_v.push_back(c_true);

    /* 
     * C False
     */
    
    automaton c_false;

    // std::string cf_symbol = "c_false" + std::to_string(i);

    c_false.alphabet.insert(init_symbol);
    c_false.alphabet.insert(tick_symbol);

    std::string c_f = "var" + std::to_string(i) + "cf", c_f0 = c_f + "0", c_f1 = c_f + "1";
    std::string Xf_prev = "var" + std::to_string(i-1) + "Xf", Xf_prev0 = Xf_prev + "0", Xf_prev1 = Xf_prev + "1";
    std::string Xf = "var" + std::to_string(i) + "Xf", Xf0 = Xf + "0", Xf1 = Xf + "1";
    std::string ack_f = "var" + std::to_string(i+1) + "ack", ack_f0 = ack_f + "0", ack_f1 = ack_f + "1";

    c_false.vars.insert(t);
    c_false.vars.insert(c_f);
    c_false.vars.insert(Xf_prev);
    c_false.vars.insert(Xf);
    c_false.vars.insert(ack_f);

    create_msat_variables(env,c_false.vars,DWA_VARIABLE_TYPE);    

    int cf_init = ct3 + 1, cf0 = cf_init + 1, cf1 = cf_init + 2, cf2 = cf_init + 3, cf3 = cf_init + 4;

    c_false.init_states.insert(cf_init);
    c_false.final_states.insert(cf0);

    create_rule(c_false.trans,env,cf_init,init_symbol,cf0, "(= " + Xf1 + " 0)");

    create_rule(c_false.trans,env,cf0,tick_symbol,cf1, "(and (<= " + t0 + " " + t1 + ") (= " + c_f1 + " " + t0 + ") (< (- " + t1 + " " + c_f1 + ") " + U + ") (= " + Xf_prev0 + " 1) (= " + ack_f0 + " 1) (= " + Xf1 + " 0))");

    create_rule(c_false.trans,env,cf1,tick_symbol,cf0, "(and (<= " + t0 + " " + t1 + ") (< (- " + t0 + " " + c_f0 + ") " + U + ") (or (= " + Xf_prev0 + " 0) (= " + ack_f0 + " 0)) (= " + Xf1 + " 0))");

    create_rule(c_false.trans,env,cf1,tick_symbol,cf2, "(and (<= " + t0 + " " + t1 + ") (< (- " + t0 + " " + c_f0 + ") " + U + ") (>= " + "(- " + t0 + " " + c_f0 + ") " + L + ") (= "+ Xf_prev0 + " 1) (= " + ack_f0 +" 1) (= " + Xf1 + " 1))");

    create_rule(c_false.trans,env,cf2,tick_symbol,cf3, "(and (<= " + t0 + " " + t1 + ") (= " + c_f1 + " " + t0 + ") (< (- " + t1 + " " + c_f1 + ") " + U + ") (= " + Xf_prev0 + " 0) (= " + ack_f0 + " 0) (= " + Xf1 + " 1))");

    create_rule(c_false.trans,env,cf3,tick_symbol,cf2, "(and (<= " + t0 + " " + t1 + ") (< (- " + t0 + " " + c_f0 + ") " + U + ") (or (= " + Xf_prev0 + " 1) (= " + ack_f0 + " 1)) (= " + Xf1 + " 1))");

    create_rule(c_false.trans,env,cf3,tick_symbol,cf0, "(and (<= " + t0 + " " + t1 + ") (< (- " + t0 + " " + c_f0 + ") " + U + ") (>= " + "(- " + t0 + " " + c_f0 + ") " + L + ") (= "+ Xf_prev0 + " 0) (= " + ack_f0 +" 0) (= " + Xf1+ " 0))");

    res_v.push_back(c_false);

    /*
     * NOR
     */

    automaton nor; 

    // std::string nor_symbol = "nor" + std::to_string(i);

    nor.alphabet.insert(init_symbol);
    nor.alphabet.insert(tick_symbol);

    std::string c_n = "var" + std::to_string(i) + "cn", c_n0 = c_n + "0", c_n1 = c_n + "1";
    std::string ack_n = "var" + std::to_string(i) + "ack", ack_n0 = ack_n + "0", ack_n1 = ack_n + "1";

    nor.vars.insert(t);
    nor.vars.insert(c_n);
    nor.vars.insert(Xt);
    nor.vars.insert(Xf);
    nor.vars.insert(ack_n);

    create_msat_variables(env,nor.vars,DWA_VARIABLE_TYPE);

    int nor_init = cf3 + 1, n0 = nor_init + 1, n1 = nor_init + 2, n2 = nor_init + 3, n3 = nor_init + 4;

    nor.init_states.insert(nor_init);
    nor.final_states.insert(n0);

    create_rule(nor.trans,env,nor_init,init_symbol,n0, "(= " + ack_n1 + " 0)");

    create_rule(nor.trans,env,n0,tick_symbol,n1, "(and (<= " + t0 + " " + t1 + ") (= " + c_n1 + " " + t0 + ") (< (- " + t1 + " " + c_n1 + ") " + U + ") (= " + Xt0 + " 0) (= " + Xf0 + " 0) (= " + Xt1 + " " + Xt0 + ") (= " + Xf1 + " " + Xf0 + ") (= " + ack_n1 + " 0))");

    create_rule(nor.trans,env,n1,tick_symbol,n0, "(and (<= " + t0 + " " + t1 + ") (< (- " + t0 + " " + c_n0 + ") " + U + ") (or (= " + Xt0 + " 1) (= " + Xf0 + " 1)) (= " + Xt1 + " " + Xt0 + ") (= " + Xf1 + " " + Xf0 + ") (= " + ack_n1 + " 0))");

    create_rule(nor.trans,env,n1,tick_symbol,n2, "(and (<= " + t0 + " " + t1 + ") (< (- " + t0 + " " + c_n0 + ") " + U + ") (>= " + "(- " + t0 + " " + c_n0 + ") " + L + ") (= " + Xt0 + " 0) (= " + Xf0 + " 0) (= " + Xt1 + " " + Xt0 + ") (= " + Xf1 + " " + Xf0 + ") (= " + ack_n1 + " 1))");

    create_rule(nor.trans,env,n2,tick_symbol,n3, "(and (<= " + t0 + " " + t1 + ") (< (- " + t1 + " " + c_n1 + ") " + U + ") (or (= " + Xt0 + " 1) (= " + Xf0 + " 1)) (= " + Xt1 + " " + Xt0 + ") (= " + Xf1 + " " + Xf0 + ") (= " + ack_n1 + " 1))");

    create_rule(nor.trans,env,n3,tick_symbol,n2, "(and (<= " + t0 + " " + t1 + ") (= " + c_n1 + " " + t0 + ") (< (- " + t0 + " " + c_n0 + ") " + U + ") (= " + Xt0 + " 0) (= " + Xf0 + " 0) (= " + Xt1 + " " + Xt0 + ") (= " + Xf1 + " " + Xf0 + ") (= " + ack_n1 + " 1))");

    create_rule(nor.trans,env,n3,tick_symbol,n0, "(and (<= " + t0 + " " + t1 + ") (< (- " + t0 + " " + c_n0 + ") " + U + ") (>= " + "(- " + t0 + " " + c_n0 + ") " + L + ") (or (= "+ Xt0 + " 1) (= " + Xf0 + " 1)) (= " + Xt1 + " " + Xt0 + ") (= " + Xf1 + " " + Xf0 + ") (= " + ack_n1 + " 0))");

    res_v.push_back(nor);
  }

  return res_v;
}


automaton monitor1(msat_env env) {
  automaton monitor;

  monitor.alphabet.insert(init_symbol);  
  monitor.alphabet.insert(tick_symbol);

  std::string Xt = "var" + std::to_string(K) + "Xt", Xt0 = Xt + "0", Xt1 = Xt + "1";
  std::string Xf = "var" + std::to_string(K) + "Xf", Xf0 = Xf + "0", Xf1 = Xf + "1";
  std::string ack = "var" + std::to_string(K+1) + "ack", ack0 = ack + "0", ack1 = ack + "1";

  monitor.vars.insert(Xt);
  monitor.vars.insert(Xf);
  monitor.vars.insert(ack);

  int m_init = 0, m0 = 1, m1 = 2, m2 = 3;

  monitor.init_states.insert(m_init);
  monitor.final_states.insert(m0);
  monitor.final_states.insert(m1);
  monitor.final_states.insert(m2);

  create_rule(monitor.trans,env,m_init,init_symbol,m0, "true");

  create_rule(monitor.trans,env,m0,tick_symbol,m0, "(and (= " + Xf0 + " 0) (= " + Xt0 + " 0))");
  create_rule(monitor.trans,env,m0,tick_symbol,m1, "(and (= " + Xf0 + " 1) (= " + Xt0 + " 0))");
  create_rule(monitor.trans,env,m1,tick_symbol,m0, "(and (= " + Xf0 + " 0) (= " + Xt0 + " 0))");
  create_rule(monitor.trans,env,m0,tick_symbol,m2, "(and (= " + Xt0 + " 1) (= " + Xf0 + " 0))");
  create_rule(monitor.trans,env,m2,tick_symbol,m0, "(and (= " + Xt0 + " 0) (= " + Xf0 + " 0))");

  return monitor;
}

automaton get_rhs_automaton_concrete(msat_env env) {
  return monitor1(env);
}
