#include "main.h"
#include "create_automata.h"

int N=5;

std::vector<automaton> get_lhs_automaton_concrete(msat_env env)
{
  std::vector<automaton> res_v;

  std::string t = "tau";
  std::string t0 = t+"0";
  std::string t1 = t+"1";
  DWA_VARIABLE_TYPE=msat_get_rational_type(env); // global type redefinition

  
  for (int i = 0; i < N; i ++) {
    automaton train;

    std::string app = "app" + std::to_string(i);
    std::string enter = "enter"+ std::to_string(i);
    std::string exit = "exit" + std::to_string(i);

    train.alphabet.insert(app);
    train.alphabet.insert(enter);
    train.alphabet.insert(exit);

    std::string x = "v"+std::to_string(i)+"x";
    std::string x0 = x+"0";
    std::string x1 = x+"1";

    train.vars.insert(t);
    train.vars.insert(x);
    create_msat_variables(env,train.vars,DWA_VARIABLE_TYPE);

    int st0=3*i;
    int st1=3*i+1;
    int st2=3*i+2;

    train.init_states.insert(st0);
    train.final_states.insert(st0);
    
    create_rule(train.trans,env,st0,app,st1,"(and (<= "+t0+" "+t1+") (= "+x1+" "+t0+") (<= (- "+t1+" "+x1+") 5))");
    create_rule(train.trans,env,st1,enter,st2,"(and (<= "+t0+" "+t1+") (= "+x1+" "+x0+") (> (- "+t0+" "+x0+") 2) (<= (- "+t0+" "+x0+") 5) (<= (- "+t1+" "+x1+") 5))");
    create_rule(train.trans,env,st2,exit,st0,"(and (<= "+t0+" "+t1+") (= "+x1+" "+x0+") (<= (- "+t0+" "+x0+") 5) (= "+x0+" (+ "+t0+" 1)))");

    res_v.push_back(train);
  }

  automaton gate;

  gate.alphabet.insert("lower");
  gate.alphabet.insert("down");
  gate.alphabet.insert("raise");
  gate.alphabet.insert("up");

  gate.vars.insert(t);
  gate.vars.insert("y");

  create_msat_variables(env,gate.vars,DWA_VARIABLE_TYPE);

  int sg0=3*N;
  int sg1=3*N+1;
  int sg2=3*N+2;
  int sg3=3*N+3;

  gate.init_states.insert(sg0);
  gate.final_states.insert(sg0);

  create_rule(gate.trans,env,sg0,"lower",sg1,"(and (<= "+t0+" "+t1+") (= y1 "+t0+") (<= (- "+t1+" y1) 1))");
  create_rule(gate.trans,env,sg1,"down",sg2,"(and (<= "+t0+" "+t1+") (<= (- "+t0+" y0) 1) (= y1 (+ "+t0+" 1)) )");
  create_rule(gate.trans,env,sg2,"raise",sg3,"(and (<= "+t0+" "+t1+") (= y1 "+t0+") (<= (- "+t1+" y1) 2))");
  create_rule(gate.trans,env,sg3,"up",sg0,"(and (<= "+t0+" "+t1+") (= y1 (+ "+t0+" 1)) (<= (- "+t0+" y0) 2)) (>= (- "+t0+" y0) 1))");

  res_v.push_back(gate);

  automaton controller;

  controller.alphabet.insert("lower");
  controller.alphabet.insert("raise");

  controller.vars.insert(t);
  controller.vars.insert("z");

  create_msat_variables(env,controller.vars,DWA_VARIABLE_TYPE);

  int sc0=3*N+4;
  int sc1=sc0+1;
  int sc2=sc0+2;
  int sc3=sc0+3;

  controller.init_states.insert(sc0);
  controller.final_states.insert(sc0);

  for (int i = 0; i < N; i ++) {
    std::string app = "app" + std::to_string(i);
    std::string exit = "exit" + std::to_string(i);
    
    controller.alphabet.insert(app);
    controller.alphabet.insert(exit);

    create_rule(controller.trans,env,sc0,app,sc1,"(and (<= "+t0+" "+t1+") (= z1 "+t0+") (<= (- "+t1+" z1) 3))");
    create_rule(controller.trans,env,sc1,"lower",sc2,"(and (<= "+t0+" "+t1+") (= z1 (+ "+t0+" 1)) (<= (- "+t0+" z0) 3))");
    create_rule(controller.trans,env,sc2,exit,sc3,"(and (<= "+t0+" "+t1+") (= z1 "+t0+") (<= (- "+t1+" z1) 1))");
    create_rule(controller.trans,env,sc3,"raise",sc0,"(and (<= "+t0+" "+t1+") (<= (- "+t0+" z0) 3)) (= z1 (+ "+t0+" 1)))");
  }

  res_v.push_back(controller);
  
  return res_v;
}

automaton get_rhs_automaton_concrete(msat_env env)
{
  automaton res;

  std::string t = "tau";
  std::string t0 = t+"0";
  std::string t1 = t+"1";

  res.vars.insert(t);
  res.vars.insert("c");
  DWA_VARIABLE_TYPE=msat_get_rational_type(env); // global type redefinition
  create_msat_variables(env,res.vars,DWA_VARIABLE_TYPE);

  res.init_states.insert(0);
  res.final_states.insert(0);

  create_rule(res.trans,env,0,"lower",0,"(and (<= "+t0+" "+t1+") (= c1 c0))");
  create_rule(res.trans,env,0,"raise",0,"(and (<= "+t0+" "+t1+") (= c1 c0))");
  create_rule(res.trans,env,0,"up",0,"(and (<= "+t0+" "+t1+") (= c1 c0))");

  create_rule(res.trans,env,1,"lower",1,"(and (<= "+t0+" "+t1+") (= c1 c0) (<= (- "+t1+" c1) 5))");
  create_rule(res.trans,env,1,"raise",1,"(and (<= "+t0+" "+t1+") (= c1 c0) (<= (- "+t1+" c1) 5))");
  create_rule(res.trans,env,1,"down",1,"(and (<= "+t0+" "+t1+") (= c1 c0) (<= (- "+t1+" c1) 5))");

  create_rule(res.trans,env,0,"down",1,"(and (<= "+t0+" "+t1+") (= c1 "+t0+") (<= (- "+t1+" c1) 5))");
  create_rule(res.trans,env,1,"up",0,"(and (<= "+t0+" "+t1+") (= c1 c0))");

  for (int i = 0; i < N; i ++) {
    std::string app = "app" + std::to_string(i);
    std::string enter = "enter"+ std::to_string(i);
    std::string exit = "exit" + std::to_string(i);

    res.alphabet.insert(app);
    res.alphabet.insert(enter);
    res.alphabet.insert(exit);

    create_rule(res.trans,env,0,app,0,"(and (<= "+t0+" "+t1+") (= c1 c0))");
    create_rule(res.trans,env,0,enter,0,"(and (<= "+t0+" "+t1+") (= c1 c0))");
    create_rule(res.trans,env,0,exit,0,"(and (<= "+t0+" "+t1+") (= c1 c0))");

    create_rule(res.trans,env,1,app,1,"(and (<= "+t0+" "+t1+") (= c1 c0) (<= (- "+t1+" c1) 5))");
    create_rule(res.trans,env,1,enter,1,"(and (<= "+t0+" "+t1+") (= c1 c0) (<= (- "+t1+" c1) 5))");
    create_rule(res.trans,env,1,exit,1,"(and (<= "+t0+" "+t1+") (= c1 c0) (<= (- "+t1+" c1) 5))");
  }

  return res;
}


