#include "main.h"
#include "create_automata.h"

std::vector<automaton> get_lhs_automaton_concrete(msat_env env) {
  automaton res;
  /* alphabet */
  res.alphabet.insert("x");
  /* variables */
  res.vars.insert("a");
  res.vars.insert("l");
  res.vars.insert("N");
  res.vars.insert("I");
  res.vars.insert("i");
  create_msat_variables(env,res.vars,msat_get_integer_type(env));
  /* init states */
  res.init_states.insert(0);
  /* final states */
  res.final_states.insert(1);
  res.final_states.insert(2);
  res.final_states.insert(3);
  res.final_states.insert(4);

  /* create rules */
  create_rule(res.trans,env,2,"x",2,"(and (= l1 l0) (= N1 N0) (= i1 i0) (= I1 (+ I0 1)) (<= I1 (- N1 1)) (<= I1 (- i1 1)) (>= a1 (+ a0 3)) ) ");
  create_rule(res.trans,env,2,"x",1,"(and (= l1 l0) (= I1 N1) (= N1 N0) (= N1 (+ I0 1)) (= i1 i0) ) ");
  create_rule(res.trans,env,0,"x",2,"(and (= I1 1) (= l1 (- N0 1)) (= I0 0) (= l1 l0) (= l1 (- N1 1)) (= i1 i0) (>= l1 (+ i1 4)) (>= i1 3) (>= a1 (+ a0 3)) ) ");
  create_rule(res.trans,env,3,"x",4,"(and (= I1 l1) (= l1 l0) (= N1 N0) (= l1 (+ I0 1)) (= i1 i0) (= a1 (+ a0 5)) (<= l1 (- N1 1)) ) ");
  create_rule(res.trans,env,3,"x",1,"(and (= I1 N1) (= N1 N0) (= l1 l0) (= N1 (+ I0 1)) (= i1 i0) ) ");
  create_rule(res.trans,env,3,"x",3,"(and (= l1 l0) (= N1 N0) (= i1 i0) (= I1 (+ I0 1)) (= a1 (+ a0 5)) (<= I1 (- l1 1)) (<= I1 (- N1 1)) ) ");
  create_rule(res.trans,env,2,"x",3,"(and (= N1 N0) (= l1 l0) (= I1 i1) (= i1 i0) (= i1 (+ I0 1)) (>= N1 (+ i1 1)) (>= a1 (+ a0 3)) ) ");
  create_rule(res.trans,env,4,"x",1,"(and (= l1 l0) (= I1 N1) (= N1 N0) (= N1 (+ I0 1)) (= i1 i0) ) ");
  create_rule(res.trans,env,4,"x",4,"(and (= l1 l0) (= N1 N0) (= i1 i0) (= I1 (+ I0 1)) (<= I1 (- N1 1)) ) ");

  add_invariant(res,env,2,"(and (= N0 (+ l0 1)) (>= i0 3) (>= l0 (+ i0 4))  (or (= I0 1) (and (>= I0 2) (<= I0 (- i0 1)))))"); 
  add_invariant(res,env,3,"(and (= N0 (+ l0 1)) (>= N0 (+ I0 1)) (or (and (>= N0 (+ i0 1)) (>= l0 (+ I0 1)) (>= I0 (+ i0 1)) (>= l0 (+ i0 4)) (>= i0 3))  (and (= i0 I0) (>= l0 (+ I0 4)) (>= I0 3))  ))");
  add_invariant(res,env,4,"(and (= l0 I0) (= N0 (+ l0 1)) (>= l0 (+ i0 4)) (>= i0 3))");
  

  std::vector<automaton> res_v;		
  res_v.push_back(res);
  return res_v;
}

automaton get_rhs_automaton_concrete(msat_env env) {
  automaton res;
  /* alphabet */
  res.alphabet.insert("x");
  /* variables */
  res.vars.insert("a");
  res.vars.insert("l");
  res.vars.insert("N");
  res.vars.insert("I");
  res.vars.insert("i");
  create_msat_variables(env,res.vars,msat_get_integer_type(env));
  /* init states */
  res.init_states.insert(0);
  /* final states */
  res.final_states.insert(1);
  res.final_states.insert(2);
  res.final_states.insert(3);
  /* create rules */
  create_rule(res.trans,env,2,"x",3,"(and (= I1 l1) (= l1 l0) (= N1 N0) (= l1 (+ I0 1)) (= i1 i0) (>= a1 (+ a0 3)) (<= l1 (- N1 1)) ) ");
  create_rule(res.trans,env,2,"x",2,"(and (= l1 l0) (= N1 N0) (= i1 i0) (= I1 (+ I0 1)) (<= I1 (- l1 1)) (<= I1 (- N1 1)) (>= a1 (+ a0 3)) ) ");
  create_rule(res.trans,env,3,"x",1,"(and (= l1 l0) (= I1 N1) (= N1 N0) (= N1 (+ I0 1)) (= i1 i0) ) ");
  create_rule(res.trans,env,3,"x",3,"(and (= l1 l0) (= N1 N0) (= i1 i0) (= I1 (+ I0 1)) (<= I1 (- N1 1)) ) ");
  create_rule(res.trans,env,0,"x",3,"(and (= I1 1) (= l0 1) (= I0 0) (= l1 1) (= N1 N0) (= i1 i0) (>= N1 2) (>= a1 (+ a0 3)) ) ");
  create_rule(res.trans,env,0,"x",1,"(and (= I1 1) (= N0 1) (= I0 0) (= l1 l0) (= N1 1) (= i1 i0) ) ");
  create_rule(res.trans,env,2,"x",1,"(and (= l1 l0) (= I1 N1) (= N1 N0) (= N1 (+ I0 1)) (= i1 i0) ) ");
  create_rule(res.trans,env,0,"x",3,"(and (= I1 1) (= I0 0) (= l1 l0) (= N1 N0) (= i1 i0) (>= N1 2) (<= l1 0) ) ");
  create_rule(res.trans,env,0,"x",2,"(and (= I1 1) (= I0 0) (= l1 l0) (= N1 N0) (= i1 i0) (>= l1 2) (>= N1 2) (>= a1 (+ a0 3)) ) ");
  return res;
}
