#include "main.h"
#include "create_automata.h"

int N=20;
int D=20;

std::vector<automaton> get_lhs_automaton_concrete(msat_env env) {
  std::vector<automaton> res_v;

  std::string init_symbol = "init";

  std::string x = "x", x0 = x + "0", x1 = x + "1";
  std::string v = "v", v0 = v + "0", v1 = v + "1";
  std::string u = "u", u0 = u + "0", u1 = u + "1";
  
  for (int i = 0; i < N; i ++) {
    automaton p;

    std::string alpha_symbol = "a" + std::to_string(i);

    p.alphabet.insert(init_symbol);
    p.alphabet.insert(alpha_symbol);

    p.vars.insert(x);
    p.vars.insert(v);
    p.vars.insert(u);

    create_msat_variables(env,p.vars,msat_get_integer_type(env));

    int s0 = 2*i, s1 = 2*i + 1;

    p.init_states.insert(s0);
    p.final_states.insert(s1);

    create_rule(p.trans,env,s0,init_symbol,s1,"(= " + x1 + " 0)");
    create_rule(p.trans,env,s1,alpha_symbol,s1,"(and (>= " + x0 + " " +  std::to_string(D*i) +") (< " + x0 + " " + std::to_string(D*(i+1)) + ") (= " + x1 + " (+ " + x0 + " 1)) (= " + v0 + " " + u0 + "))");

    res_v.push_back(p);
  }

  return res_v;
}

automaton get_rhs_automaton_concrete(msat_env env) {
  automaton m;

  std::string init_symbol = "init";

  m.alphabet.insert(init_symbol);

  std::string x = "x", x0 = x + "0", x1 = x + "1";
  std::string v = "v", v0 = v + "0", v1 = v + "1";
  std::string u = "u", u0 = u + "0", u1 = u + "1";

  m.vars.insert(x);
  m.vars.insert(v);
  m.vars.insert(u);

  create_msat_variables(env,m.vars,msat_get_integer_type(env));

  m.init_states.insert(0);
  m.final_states.insert(1);

  create_rule(m.trans,env,0,init_symbol,1, "(= " + x1 + " 0)");

  for (int i = 0; i < N; i ++) {
    std::string alpha_symbol = "a" + std::to_string(i);

    m.alphabet.insert(alpha_symbol);

    create_rule(m.trans,env,1,alpha_symbol,1, "(= " + v0 + " " + u0 + ")" /* "(and (= " + x1 + " (+ " + x0 + " " + " 1)) (= " + v0 + " " + u0 + "))"*/);
  }

  return m;
}
