#include "main.h"
#include "create_automata.h"

int N=2;
int D=10;

std::vector<automaton> get_lhs_automaton_concrete(msat_env env) {
  std::vector<automaton> res_v;

  std::string init_symbol = "init";

  for (int i = 0; i < N; i ++) {
    automaton p;

    std::string alpha_i = "a" + std::to_string(i);

    p.alphabet.insert(init_symbol);
    p.alphabet.insert(alpha_i);

    std::string x_i = "var" + std::to_string(i) + "x", x_i0 = x_i + "0", x_i1 = x_i + "1";
    std::string v_i = "var" + std::to_string(i) + "v", v_i0 = v_i + "0", v_i1 = v_i + "1";

    p.vars.insert(x_i);
    p.vars.insert(v_i);

    create_msat_variables(env,p.vars,msat_get_integer_type(env));

    int s0 = 2*i, s1 = s0 + 1;

    p.init_states.insert(s0);
    p.final_states.insert(s1);

    create_rule(p.trans,env,s0,init_symbol,s1, "(= " + x_i1 + " 0)");
    create_rule(p.trans,env,s1,alpha_i,s1, "(and (< " + x_i0 + " " + std::to_string(D) + " ) (= " + x_i1 + " (+ " + x_i0 + " 1)) (= " + v_i1 + " (+ " + v_i0 + " " + std::to_string(i) + ")))");

    res_v.push_back(p);
  }
  
  std::string x = "x", x0 = x + "0", x1 = x + "1";
  std::string v = "v", v0 = v + "0", v1 = v + "1";
  
  for (int i = 0; i < N; i ++) {
    automaton r;

    std::string alpha_i = "a" + std::to_string(i);

    r.alphabet.insert(init_symbol);
    r.alphabet.insert(alpha_i);

    std::string v_i = "var" + std::to_string(i) + "v", v_i0 = v_i + "0", v_i1 = v_i + "1";
    std::string vnext_i = "var" + std::to_string(i+1) + "v", vnext_i0 = vnext_i + "0", vnext_i1 = vnext_i + "1";

    r.vars.insert(x);
    r.vars.insert(v);
    r.vars.insert(v_i);
    r.vars.insert(vnext_i);
    create_msat_variables(env,r.vars,msat_get_integer_type(env));

    int q0 = 3*i + 2*N, q1 = q0 + 1, q2 = q0 +2;

    r.init_states.insert(q0);
    r.final_states.insert(q2);

    create_rule(r.trans,env,q0,init_symbol,q1, "(= 0 " + x1 + ")");
    create_rule(r.trans,env,q1,alpha_i,q1, "(and (>= " + x0 + " " +  std::to_string(D*i) +") (< " + x0 + " " + std::to_string(D*(i+1)) + ") (= " + x1 + " (+ " + x0 + " 1)) (= " + v0 + " " + v_i0 + "))");
    create_rule(r.trans,env,q1,alpha_i,q2, "(and (= " + x0 + " " + std::to_string(D*(i+1)) + ") (< " + v_i0 + " " + vnext_i0 + "))");

    res_v.push_back(r);
  }

  return res_v;
}

automaton get_rhs_automaton_concrete(msat_env env) {
  automaton m;
  
  std::string init_symbol = "init";

  m.alphabet.insert(init_symbol);

  std::string x = "x", x0 = x + "0", x1 = x + "1";
  std::string v = "v", v0 = v + "0", v1 = v + "1";

  m.vars.insert(x);
  m.vars.insert(v);

  create_msat_variables(env,m.vars,msat_get_integer_type(env));

  m.init_states.insert(0);
  m.final_states.insert(1);

  create_rule(m.trans,env,0,init_symbol,1, "true");

  for (int i = 0; i < N; i ++) {
    std::string alpha_i = "a" + std::to_string(i);

    m.alphabet.insert(alpha_i);

    create_rule(m.trans,env,1,alpha_i,1, "(and (> " + x1 + " " + x0 + ") (<= " + v0 + " " + v1 + "))");
  }

  return m;
}



