#include "main.h"
#include "create_automata.h"

int get_states_no(automaton aut) {
	/* get number of states */
	int top_no=-1;
	for(std::set<int>::iterator it=aut.init_states.begin(); it!=aut.init_states.end(); ++it){
		if (*it>top_no) top_no=*it;
	}
	for(std::set<int>::iterator it=aut.final_states.begin(); it!=aut.final_states.end(); ++it){
		if (*it>top_no) top_no=*it;
	}
	a_lhs aux_lhs;
	a_rhs aux2_rhs;
	std::vector<a_rhs> aux_rhs;
	for(std::map<a_lhs,std::vector<a_rhs>>::iterator it=aut.trans.begin(); it!=aut.trans.end();++it) {
		aux_lhs=it->first;
		if (aux_lhs.src>top_no) top_no=aux_lhs.src;
		aux_rhs=it->second;
		for(std::vector<a_rhs>::iterator it2=aux_rhs.begin(); it2!=aux_rhs.end();++it2) {
			aux2_rhs=*it2;
			if (aux2_rhs.dest>top_no) top_no=aux2_rhs.dest;
		}
	}
		

	assert(top_no>=0);
	return top_no+1;
}

int get_min_states_no(automaton aut) {
	/* states are usually numbered from 0. But in some automata, some of the states may be missing.
	 * This function returns the minimum */
	int min_no=-1;
	for(std::set<int>::iterator it=aut.init_states.begin(); it!=aut.init_states.end(); ++it){
		if ((*it<min_no)||(min_no==-1)) min_no=*it;
	}
	assert(min_no>=0); // check that there is at least one initial state
	for(std::set<int>::iterator it=aut.final_states.begin(); it!=aut.final_states.end(); ++it){
		if (*it<min_no) min_no=*it;
	}
	a_lhs aux_lhs;
	a_rhs aux2_rhs;
	std::vector<a_rhs> aux_rhs;
	for(std::map<a_lhs,std::vector<a_rhs>>::iterator it=aut.trans.begin(); it!=aut.trans.end();++it) {
		aux_lhs=it->first;
		if (aux_lhs.src<min_no) min_no=aux_lhs.src;
		aux_rhs=it->second;
		for(std::vector<a_rhs>::iterator it2=aux_rhs.begin(); it2!=aux_rhs.end();++it2) {
			aux2_rhs=*it2;
			if (aux2_rhs.dest<min_no) min_no=aux2_rhs.dest;
		}
	}
		

	return min_no;
}

/* declare the variables needed in the formulae on the automata edges */
void create_msat_variables(msat_env env, std::set<std::string> vars,msat_type type){
	int i;
	msat_decl d;
	std::string v1,v2,v3;
	for (std::set<std::string>::iterator it=vars.begin(); it!=vars.end(); ++it){
		/* for each variable "v" create "v0" and "v1" in the mathsat enviroment */
		v1=*it+"0";
		v2=*it+"1";
		/* first check whether the declaration exists */
		d=msat_find_decl(env,v1.c_str());
		if(MSAT_ERROR_DECL(d)) {
			/* does not exists -> create */
			d = msat_declare_function(env, v1.c_str(), type);
        		assert(!MSAT_ERROR_DECL(d));
		}
		d=msat_find_decl(env,v2.c_str());
		if(MSAT_ERROR_DECL(d)) {
			/* does not exists -> create */
			d = msat_declare_function(env, v2.c_str(), type);
        		assert(!MSAT_ERROR_DECL(d));
		}
	}

}

/* create an automata rule */
void create_rule(std::map<a_lhs,std::vector<a_rhs>> &trans, msat_env env, int src_state, std::string symbol, int dst_state, std::string formula){
	a_lhs lhs;
	a_rhs rhs;
	lhs.src=src_state;
	lhs.lab=symbol;
	rhs.dest=dst_state;
	rhs.formula=msat_from_string(env,formula.c_str());
	if MSAT_ERROR_TERM(rhs.formula) {
		printf("Unpossible to convert %s to mathsat term\n",formula.c_str());
		assert(0);
	}
	if (DWA_VERBOSE)
		std::cout << "Adding automata rule: "<< lhs.src <<" -- "<<lhs.lab<<", "<<msat_term_repr(rhs.formula)<<" --> "<<rhs.dest<<"\n";
	int count=0; // count the occurences of rhs.dest in trans[lhs]. Correct is 0, or 1
	// the tool relies on the fact, that there is only a single transition q1 -- x --> q2
	// therefore if there is more then one in the input, we have to make a disjunction of formula 
	for(std::vector<a_rhs>::iterator i=trans[lhs].begin(); i!=trans[lhs].end();++i){
		//a_rhs tmp=*i;
		if ((*i).dest==rhs.dest) {
			(*i).formula=msat_make_or(env,(*i).formula,rhs.formula);
			count++;
		}
	}
	assert(count<2);
	if(!count) 
		trans[lhs].push_back(rhs);
}

void add_invariant(automaton &aut, msat_env env, int state, std::string inv) {
	aut.st_invariant[state]=msat_from_string(env,inv.c_str());
	if MSAT_ERROR_TERM(aut.st_invariant[state]) {
		printf("Unpossible to convert %s to mathsat term\n",inv.c_str());
		assert(0);
	}

}

void get_alpha_vars(product_automaton &lhs) {
	for(int i=0; i<lhs.components.size();i++) {
		for(std::set<std::string>::iterator j=lhs.components[i].alphabet.begin();j!=lhs.components[i].alphabet.end();++j) 
			lhs.alphabet.insert(*j);
		for(std::set<std::string>::iterator j=lhs.components[i].vars.begin();j!=lhs.components[i].vars.end();++j) 
			lhs.vars.insert(*j);
	}
}

product_automaton get_lhs_automaton(msat_env env){
	product_automaton res;
	res.components=get_lhs_automaton_concrete(env);
	get_alpha_vars(res);
	return res;
}

automaton get_rhs_automaton(msat_env env){
	return get_rhs_automaton_concrete(env);
}

