#include "main.h"
#include "create_automata.h"


std::vector<automaton> get_lhs_automaton_concrete(msat_env env){
	automaton res;
	/* alphabet */
	res.alphabet.insert("a");
	res.alphabet.insert("r");
	/* variables */
	res.vars.insert("a");
	res.vars.insert("m");
	create_msat_variables(env,res.vars,msat_get_integer_type(env));
	/* init states */
	res.init_states.insert(0);
	/* final states */
	res.final_states.insert(1);
	res.final_states.insert(2);
	res.final_states.insert(3);
	res.final_states.insert(4);
	res.final_states.insert(5);
	/* create rules */
	create_rule(res.trans,env,0,"a",1,"(and (> m0 1) (= m1 m0) (= a1 0))");
	//t0: init_p -> s1 {m > 1 && m'=m && a' = 0}
	create_rule(res.trans,env,2,"a",4,"(and (< a0 (- m0 1)) (= a1 (+ a0 1)) (= m0 m1))");
	//t3: s2 -> s4 {a != m - 1 && a' = a + 1 && m' = m}
	create_rule(res.trans,env,2,"r",4,"(and (= a0 (- m0 1)) (= a1 0) (= m0 m1))");
	//t4: s2 -> s4 {a = m - 1 && a' = 0 && m' = m}
	//t5: s2 -> s4 {a != m - 1 && a = m - 1 && a' = a + 1 && a' = 0 && m' = m}
	create_rule(res.trans,env,2,"a",1,"(and (= a1 0) (= m0 m1))");
	//t6: s2 -> s1 {a' = 0 && m' = m}
	create_rule(res.trans,env,2,"a",5,"(and (= a0 m0) (= a1 a0) (= m0 m1))");
	//t7: s2 -> bad {a = m && a' = a && m' = m}
	create_rule(res.trans,env,4,"a",2,"(and (= a1 a0) (= m0 m1))");
	//t8: s4 -> s2 {a' = a && m' = m}
	create_rule(res.trans,env,4,"a",3,"(and (= a1 0) (= m0 m1))");
	//t9: s4 -> s3 {a' = 0 && m' = m}
	create_rule(res.trans,env,4,"a",5,"(and (= a0 m0) (= a1 a0) (= m0 m1))");
	//t10: s4 -> bad {a = m && a' = a && m' = m}
	create_rule(res.trans,env,1,"a",2,"(and (= a1 a0) (= m0 m1))");
	//t11: s1 -> s2 {a' = a && m' = m}
	create_rule(res.trans,env,1,"a",3,"(and (= a1 0) (= m0 m1))");
	//t12: s1 -> s3 {a' = 0 && m' = m}
	create_rule(res.trans,env,3,"a",4,"(and (= a1 a0) (= m0 m1))");
	//t13: s3 -> s4 {a' = a && m' = m}
	create_rule(res.trans,env,3,"a",1,"(and (= a1 0) (= m0 m1))");
	//t14: s3 -> s1 {a' = 0 && m' = m}
	add_invariant(res,env,1,"(and (> m0 a0) (> m0 1))");
	add_invariant(res,env,2,"(and (> m0 a0) (> m0 1))");
	add_invariant(res,env,3,"(and (> m0 a0) (> m0 1))");
	add_invariant(res,env,4,"(and (> m0 a0) (> m0 1))");

	std::vector<automaton> res_v;		
	res_v.push_back(res);
	return res_v;

}


automaton get_rhs_automaton_concrete(msat_env env){
	automaton res;
	/* alphabet */
	res.alphabet.insert("a");
	/* variables */
	res.vars.insert("a");
	res.vars.insert("m");
	create_msat_variables(env,res.vars,msat_get_integer_type(env));
	/* init states */
	res.init_states.insert(0);
	/* final states */
	res.final_states.insert(0);
	/* create rules */
	create_rule(res.trans,env,0,"a",0,"(< a1 m1)");
	create_rule(res.trans,env,0,"r",0,"(< a1 m1)");


	return res;

}
