#include "main.h"


int compute_interpolants(msat_env env, product_state *end, int seq_begin,std::set<std::string> vars) {
	/* Refine the path with the last node "end" and the first node with seq=seq_begin
	 * The path must be unsatifiable
	 *
	 * return 1: succesfull refinement
	 * return 0: not succesfull, the considered path from "seq_begin" to "end" is satisfiable. Try with lower seq_begin.
	 */
	int group_a, group_b,res;
	msat_result result;	
	int groups[1];	
	std::cout << "Computing interpolants\n";
	assert(end->seq>(seq_begin+1)); // path must be at least of lenght 2. Othervice it does not make sense to compute interpolants
	assert(seq_begin>=0);
	product_state *mid,*aux;
	msat_term ip_form_left,ip_form_right;
	mid=end->prev;
	while(mid->seq>seq_begin){
		aux=end;
		ip_form_right=msat_make_true(env);
		while(aux!=mid) {
			// prepare map to rename trans according to the seq
			std::map<std::string,std::string> var_map;
			for(std::set<std::string>::iterator i=vars.begin();i!=vars.end();++i){ 
				// prepare variable remapping for apply_trans 
				std::string seq;
				char seqtmp[50];
				snprintf(seqtmp,50,"%i", (aux->seq)); seq=seqtmp;
				var_map[*i + "1"]=*i + seq;
				AddMissingVar(env,*i + seq,DWA_VARIABLE_TYPE);
				snprintf(seqtmp,50,"%i", ((aux->seq)-1)); seq=seqtmp;
				var_map[*i + "0"]=*i + seq;
				AddMissingVar(env,*i + seq,DWA_VARIABLE_TYPE);
			}
			ip_form_right=msat_make_and(env,ip_form_right,term_rename(env,aux->trans,var_map));
			aux=aux->prev;
		}
		ip_form_left=msat_make_true(env);
		while(aux->seq>seq_begin) {
			// prepare map to rename trans according to the seq
			std::map<std::string,std::string> var_map;
			for(std::set<std::string>::iterator i=vars.begin();i!=vars.end();++i){ 
				// prepare variable remapping for apply_trans 
				std::string seq;
				char seqtmp[50];
				snprintf(seqtmp,50,"%i", (aux->seq)); seq=seqtmp;
				var_map[*i + "1"]=*i + seq;
				AddMissingVar(env,*i + seq,DWA_VARIABLE_TYPE);
				snprintf(seqtmp,50,"%i", ((aux->seq)-1)); seq=seqtmp;
				var_map[*i + "0"]=*i + seq;
				AddMissingVar(env,*i + seq,DWA_VARIABLE_TYPE);
			}
			ip_form_left=msat_make_and(env,ip_form_left,term_rename(env,aux->trans,var_map));
			aux=aux->prev;
		}
		// aux->abs is the initial state - must be added to ip_form_left
		std::map<std::string,std::string> var_map,var_map_itp;
		for(std::set<std::string>::iterator i=vars.begin();i!=vars.end();++i){ 
			// prepare variable remapping for apply_trans 
			std::string seq;
			char seqtmp[50];
			snprintf(seqtmp,50,"%i", (aux->seq)); seq=seqtmp;
			var_map[*i + "0"]=*i + seq;
			AddMissingVar(env,*i + seq,DWA_VARIABLE_TYPE);
			snprintf(seqtmp,50,"%i", (mid->seq)); seq=seqtmp;
			var_map_itp[*i + seq]=*i + "1";
			AddMissingVar(env,*i + "1",DWA_VARIABLE_TYPE);
		}
		ip_form_left=msat_make_and(env,ip_form_left,term_rename(env,aux->abs,var_map));
		assert(aux->seq==seq_begin);
	
		//new we compute the interpolant between ip_form_left and ip_form_right
		msat_term itp;
		res=msat_reset_env(env); /* clear the assertions from the enviriment */
		assert(res==0);
		group_a = msat_create_itp_group(env);
		group_b = msat_create_itp_group(env);
		res = msat_set_itp_group(env, group_a);
		assert(res == 0);
		res = msat_assert_formula(env,ip_form_left); 
		assert(res == 0);
		res = msat_set_itp_group(env, group_b);
		assert(res == 0);
		res = msat_assert_formula(env,ip_form_right); 
		assert(res == 0);
		result=msat_solve(env);
		// the path is satisfiable -> no refinement posible. Return 0
		if(result==MSAT_SAT) return 0;
		// path is unsatisfiable, refine the abstraction
		groups[0]=group_a;
		itp=msat_get_interpolant(env,groups,1);
		assert(!MSAT_ERROR_TERM(itp));
		// rename all variables in itp from "var<mid->seq>" to "var1"
		itp=term_rename(env,itp,var_map_itp);
		if(check_sat(env,itp)) {
			std::cout << "Adding interpolant: "<<msat_term_repr(itp)<<"\n";
			apply_trans_settings.predicates.push_back(itp);
		}else{
			std::cout << "Interpolant = false. Nothing added\n";
		}
		mid=mid->prev; //continue with the next element
	}
	return 1;

}
