/* Adam Rogalewicz
 * INCLUDER project
 * distributed under GNU GPL v3 */

#include <map>
#include <string>
#include "mathsat.h"
#include <utility>
#include <set>
#include<vector>
#include<queue>
#include<assert.h>
#include <iostream>
#include<ctime>

// mathsat with existential quantifier elimination is available
#define USE_MSAT_EQ
#ifdef USE_MSAT_EQ
#include "msatexistelim.h"
#endif


// Global variables
extern int DWA_VERBOSE;
extern msat_type DWA_VARIABLE_TYPE;

typedef struct {
	std::map<std::string,std::string> var_map;
	msat_term *qvars;
	int size;
	std::vector<msat_term> predicates;
} TRANS_SETTING;

extern TRANS_SETTING apply_trans_settings;

/* types to store automata */
typedef struct {
	int src;
	std::string lab;
} a_lhs;

typedef struct {
	int dest;
	msat_term formula;
} a_rhs;

bool operator==(a_lhs const left, a_lhs const right);
bool operator<(a_lhs const left, a_lhs const right);

typedef struct {
	std::set<std::string> alphabet;
	std::set<std::string> vars;
	std::set<int> init_states;
	std::set<int> final_states;
	std::map<a_lhs,std::vector<a_rhs>> trans;
	std::map<int,msat_term> st_invariant;
} automaton;

typedef struct {
	std::set<std::string> alphabet;
	std::set<std::string> vars;
	std::vector<automaton> components;
} product_automaton;




/* product state */
typedef struct p_stateT {
	int seq;
	std::vector<int> l; /* on the lhs, we work with on-the-fly product */
	std::set<int> r;
	msat_term abs,concr; /* abstract and concrete formula */
	msat_term trans; /* the applied transition */
	struct p_stateT *prev=NULL, *subsumed_by=NULL;
	std::vector<struct p_stateT*> next, subsume;
	std::string symbol;
} product_state;

std::ostream& operator<< (std::ostream& os, const product_state a);


/* Term rename */
msat_term term_rename(msat_env env, msat_term orig, std::map<std::string,std::string> var_map);
int term_size(msat_env env, msat_term orig);

/* compute simulations */

typedef struct {
	int state;
	std::string symbol;
	msat_term form;
} pre_post_item;

bool operator==(pre_post_item  const left, pre_post_item const right);
bool operator<(pre_post_item const left, pre_post_item const right);

typedef struct {
	std::vector<std::vector<msat_term>> sim;
	int cor; // sim[i][j] refers to the automata states i+correction and j+correction (states [0 - correction-1] unused in the automaton)
	int rhs; // in the case of cross simulations, the rhs begins at this label
	int time; // time needet to compute the simulation
} sim_item;

typedef struct {
	std::vector<sim_item> lhs;
	sim_item rhs;
} simulations;

simulations compute_simulation(msat_env env, product_automaton lhs, automaton rhs, int limit, int size_limit);
simulations compute_cross_simulation(msat_env env, product_automaton lhs, automaton rhs, int limit, int size_limit);
int apply_cross_sim(msat_env env, product_state *s, simulations sim);

/* subsumption bussiness */


/* prototypes of functions from dwa_inclusion.c */
void AddMissingVar(msat_env env, std::string v1,msat_type type);

int check_term_implication(msat_env env, msat_term t1, msat_term t2, int no_reset);
int check_sat(msat_env env, msat_term t);
int check_term_equivalence(msat_env env, msat_term t1, msat_term t2);

std::set<std::string> str_set_union(std::set<std::string> v1, std::set<std::string> v2);

/* automata input */

product_automaton get_lhs_automaton(msat_env env);

automaton get_rhs_automaton(msat_env env);

int get_states_no(automaton aut);
int get_min_states_no(automaton aut);

/* QE bypass */
msat_term msat_exist_elim_bypass(msat_env env, msat_term formula,
                          msat_term *qvars, size_t num_vars_to_elim,
                          msat_exist_elim_algorithm algo);

/* interpolation */
int compute_interpolants(msat_env env, product_state *end,int seq_begin,std::set<std::string> vars);

