#include "main.h"
#include "create_automata.h"

std::string Bufdelay="2";
std::string Limitsend="5";
std::string Datalimit="20";


std::vector<automaton> get_lhs_automaton_concrete(msat_env env) {
  /* sender */
  automaton sender;
  sender.alphabet.insert("init");
  sender.alphabet.insert("s");
  sender.alphabet.insert("send_data");
  sender.alphabet.insert("rec_ack");
  sender.vars.insert("sbit");
  sender.vars.insert("sdata");
  sender.vars.insert("bdata");
  sender.vars.insert("bbit");
  sender.vars.insert("sack");
  sender.vars.insert("xs");
  sender.vars.insert("t");
  create_msat_variables(env,sender.vars,msat_get_integer_type(env));
  sender.init_states.insert(0);
  sender.final_states.insert(1);
  create_rule(sender.trans, env, 0, "init", 1, "(and (= sbit1 0) (= sdata1 0))" );
  create_rule(sender.trans, env, 1, "s", 2, "(and (= sbit0 sbit1) (<= t0 t1) (< sdata0 "+Datalimit+") (= sdata1 (+ sdata0 1)))" );
  create_rule(sender.trans, env, 1, "s", 2, "(and (= sbit0 sbit1) (<= t0 t1) (>= sdata0 "+Datalimit+") (= sdata1 0))" );
  create_rule(sender.trans, env, 2, "send_data", 3, "(and (= sbit0 sbit1) (= bbit1 sbit0) (<= t0 t1) (= xs1 t0) (= sdata0 sdata1) (= bdata1 sdata0))" );
  create_rule(sender.trans, env, 3, "s", 2, "(and (= sbit0 sbit1) (<= t0 t1) (<= " + Limitsend + " (- t0 xs0)) (= sdata0 sdata1))" );
  create_rule(sender.trans, env, 3, "rec_ack", 4, "(and (= sbit0 sbit1) (<= t0 t1) (= sdata0 sdata1))" );
  create_rule(sender.trans, env, 4, "s", 1, "(and (= sbit0 sack0) (<= t0 t1) (= sbit1 (- 1 sbit0)) (= sdata0 sdata1))" );


  /* receiver */
  automaton receiver;
  receiver.alphabet.insert("init");
  receiver.alphabet.insert("rec_data");
  receiver.alphabet.insert("send_ack");
  receiver.vars.insert("rbit");
  receiver.vars.insert("back");
  receiver.vars.insert("t");
  receiver.vars.insert("rdata");
  create_msat_variables(env,receiver.vars,msat_get_integer_type(env));
  receiver.init_states.insert(0);
  receiver.final_states.insert(1);
  create_rule(receiver.trans, env, 0, "init", 1, "true" );
  create_rule(receiver.trans, env, 1, "rec_data", 2, "true" );
  create_rule(receiver.trans, env, 2, "send_ack", 1, "(and (= back1 rbit0) (<= t0 t1))" );



 /* buffer */
  automaton buffer;
  buffer.alphabet.insert("init");
  buffer.alphabet.insert("b");
  buffer.alphabet.insert("send_data");
  buffer.alphabet.insert("rec_ack");
  buffer.alphabet.insert("rec_data");
  buffer.alphabet.insert("send_ack");
  buffer.vars.insert("bbit");
  buffer.vars.insert("rbit");
  buffer.vars.insert("bdata");
  buffer.vars.insert("rdata");
  buffer.vars.insert("back");
  buffer.vars.insert("sack");
  buffer.vars.insert("xb");
  buffer.vars.insert("t");
  create_msat_variables(env,buffer.vars,msat_get_integer_type(env));
  buffer.init_states.insert(0);
  buffer.final_states.insert(1);
  create_rule(buffer.trans, env, 0, "init", 1, "true" );
  create_rule(buffer.trans, env, 1, "send_data", 3, "(and (<= t0 t1) (= xb1 t0))" );
  create_rule(buffer.trans, env, 3, "rec_data", 1, "(and (<= t0 t1) (= rbit1 bbit0) (<= "+ Bufdelay +" (- t0 xb0)) (= rdata1 bdata0))" );
  create_rule(buffer.trans, env, 3, "b", 1, "(and (<= t0 t1) (<= " + Bufdelay + " (- t0 xb0)))" );
  create_rule(buffer.trans, env, 1, "send_ack", 2, "(and (<= t0 t1) (= xb1 t0))" );
  create_rule(buffer.trans, env, 2, "rec_ack", 1, "(and (<= t0 t1) (= sack1 back0) (<= "+ Bufdelay +" (- t0 xb0)))" );
  create_rule(buffer.trans, env, 2, "b", 1, "(and (<= t0 t1) (<= " + Bufdelay + " (- t0 xb0)))" );



  std::vector<automaton> res_v;		
  res_v.push_back(sender);
  res_v.push_back(receiver);
  res_v.push_back(buffer);
  return res_v;
}

automaton get_rhs_automaton_concrete(msat_env env) {
  automaton res;
  /* alphabet */
  res.alphabet.insert("tau");
  res.alphabet.insert("send_data");
  res.alphabet.insert("rec_data");
  res.alphabet.insert("send_ack");
  res.alphabet.insert("rec_ack");
  /* variables */
  res.vars.insert("sbit");
  res.vars.insert("bbit");
  res.vars.insert("sack");
  create_msat_variables(env,res.vars,msat_get_integer_type(env));
  /* init states */
  res.init_states.insert(0);
  /* final states */
  res.final_states.insert(1);
  /* create rules */
  create_rule(res.trans, env, 0, "init", 1, "true" );
  create_rule(res.trans, env, 1, "s", 1, "true" );
  create_rule(res.trans, env, 1, "b", 1, "true" );
  create_rule(res.trans, env, 1, "send_data", 2, "(= sbit0 bbit1)" );

  create_rule(res.trans, env, 2, "s", 2, "true" );
  create_rule(res.trans, env, 2, "b", 2, "true" );
  create_rule(res.trans, env, 2, "send_data", 2, "true" );
  create_rule(res.trans, env, 2, "rec_data", 2, "true" );
  create_rule(res.trans, env, 2, "send_ack", 2, "true" );
  create_rule(res.trans, env, 2, "rec_ack", 1, "(= sbit0 sack1)" ); //check

  return res;
}
