#include "main.h"
#include "create_automata.h"

#define __P1_01 "(and (<= t0 t1) (= x1 t0) (<= t1 (+ x1 D0)) (= id0 0) (= id1 id0) (= cs1 cs0) (= D1 D0) (= y1 y0))"
#define __P1_12 "(and (<= t0 t1) (= x1 t0) (<= t0 (+ x0 D0)) (= id1 1) (= cs1 cs0) (= D1 D0) (= y1 y0))"
#define __P1_21 "(and (<= t0 t1) (= x1 t0) (<= t1 (+ x1 D0)) (= id0 0) (= id1 id0) (= cs1 cs0) (= D1 D0) (= y1 y0))"
#define __P1_23 "(and (<= t0 t1) (> t0 (+ x0 D0)) (= id0 1) (= id1 id0) (= cs1 (+ cs0 1)) (= D1 D0) (= x1 x0) (= y1 y0))"
#define __P1_30 "(and (<= t0 t1) (= id1 0) (= cs1 (- cs0 1)) (= D1 D0) (= x1 x0) (= y1 y0))"

#define __P2_01 "(and (<= t0 t1) (= y1 t0) (<= t1 (+ y1 D0)) (= id0 0) (= id1 id0) (= cs1 cs0) (= D1 D0) (= x1 x0))"
#define __P2_12 "(and (<= t0 t1) (= y1 t0) (<= t0 (+ y0 D0)) (= id1 2) (= cs1 cs0) (= D1 D0) (= x1 x0))"
#define __P2_21 "(and (<= t0 t1) (= y1 t0) (<= t1 (+ y1 D0)) (= id0 0) (= id1 id0) (= cs1 cs0) (= D1 D0) (= x1 x0))"
#define __P2_23 "(and (<= t0 t1) (> t0 (+ y0 D0)) (= id0 2) (= id1 id0) (= cs1 (+ cs0 1)) (= D1 D0) (= x1 x0) (= y1 y0))"
#define __P2_30 "(and (<= t0 t1) (= id1 0) (= cs1 (- cs0 1)) (= D1 D0) (= x1 x0) (= y1 y0))"

std::vector<automaton> get_lhs_automaton_concrete(msat_env env) {
  automaton res;

  /* alphabet */
  res.alphabet.insert("tau");
  /* variables */
  res.vars.insert("t");
  res.vars.insert("id");
  res.vars.insert("x");
  res.vars.insert("y");
  res.vars.insert("D");
  res.vars.insert("cs");
  create_msat_variables(env,res.vars,msat_get_integer_type(env));
  /* init states */
  res.init_states.insert(0);
  /* final states */
  res.final_states.insert(1);
  /* create rules */
  create_rule(res.trans,env,0,"tau",1,"(and (= t1 0) (= id1 0) (= x1 0) (= y1 0) (< 0 D1) (= cs1 0))");

  for (int i = 0; i < 4; i ++) {
    create_rule(res.trans,env,1+4*i,"tau",2+4*i,__P2_01);
    create_rule(res.trans,env,2+4*i,"tau",3+4*i,__P2_12);
    create_rule(res.trans,env,3+4*i,"tau",2+4*i,__P2_21);
    create_rule(res.trans,env,3+4*i,"tau",4+4*i,__P2_23);
    create_rule(res.trans,env,4+4*i,"tau",1+4*i,__P2_30);

    create_rule(res.trans,env,1+i,"tau",5+i,__P1_01);
    create_rule(res.trans,env,5+i,"tau",9+i,__P1_12);
    create_rule(res.trans,env,9+i,"tau",5+i,__P1_21);
    create_rule(res.trans,env,9+i,"tau",13+i,__P1_23);
    create_rule(res.trans,env,13+i,"tau",1+i,__P1_30);
  }

  add_invariant(res,env,1,"(= cs0 0)");
  add_invariant(res,env,2,"(= cs0 0)");
  add_invariant(res,env,3,"(= cs0 0)");
  add_invariant(res,env,4,"(= cs0 1)");

  add_invariant(res,env,5,"(= cs0 0)");
  add_invariant(res,env,6,"(= cs0 0)");
  add_invariant(res,env,7,"(= cs0 0)");
  add_invariant(res,env,8,"(= cs0 1)");

  add_invariant(res,env,9,"(= cs0 0)");
  add_invariant(res,env,10,"(= cs0 0)");
  add_invariant(res,env,11,"(= cs0 0)");
  add_invariant(res,env,12,"(= cs0 1)");

  add_invariant(res,env,13,"(= cs0 1)");
  add_invariant(res,env,14,"(= cs0 1)");
  add_invariant(res,env,15,"(= cs0 1)");
  add_invariant(res,env,16,"(= cs0 2)");
  std::vector<automaton> res_v;		
  res_v.push_back(res);
  return res_v;


}

automaton get_rhs_automaton_concrete(msat_env env) {
  automaton res;

  /* alphabet */
  res.alphabet.insert("tau");
  /* variables */
  res.vars.insert("cs");
  create_msat_variables(env,res.vars,msat_get_integer_type(env));
  /* init states */
  res.init_states.insert(0);
  /* final states */
  res.final_states.insert(1);
  /* create rules */
  create_rule(res.trans,env,0,"tau",1,"(= cs1 0)");

  create_rule(res.trans,env,1,"tau",1,"(= cs1 cs0)");
  create_rule(res.trans,env,1,"tau",2,"(= cs1 (+ cs0 1))");
  create_rule(res.trans,env,2,"tau",2,"(= cs1 cs0)");
  create_rule(res.trans,env,2,"tau",1,"(= cs1 (- cs0 1))");

  return res;
}
