#include "main.h"
#include "create_automata.h"

#define __P12 "(and (= ic0 BS0) (= ic1 ic0) (= BS1 BS0))"
#define __P13 "(and (< ic0 BS0) (= ic1 (+ ic0 1)) (= BS1 BS0))"
#define __P22 "(and (= ic1 ic0) (= BS1 BS0))"
#define __P23 "(and (= ic1 (+ ic0 1)) (= BS1 BS0))"
#define __P31_1 "(and (= ic0 1) (= ic1 ic0) (= BS1 BS0))"
#define __P31_2 "(and (< 1 ic0) (= ic1 ic0) (= BS1 BS0))"

#define __C12 "(and (= ic0 0) (= ic1 ic0) (= BS1 BS0))"
#define __C13 "(and (< 0 ic0) (= ic1 (- ic0 1)) (= BS1 BS0))"
#define __C22 "(and (= ic1 ic0) (= BS1 BS0))"
#define __C23 "(and (= ic1 (- ic0 1)) (= BS1 BS0))"
#define __C31_1 "(and (= ic0 (- BS0 1)) (= BS1 BS0))"
#define __C31_2 "(and (< ic0 (- BS0 1)) (= ic1 ic0) (= BS1 BS0))"

#define __P23_1__C31 "(and (= ic1 (+ ic0 1)) (= ic0 (- BS0 1)) (= BS1 BS0))"
#define __P31_1__C23 "(and (= ic0 1) (= ic1 (- ic0 1)) (= BS1 BS0))"

#define __SAFETY "(and (<= 0 ic0) (< ic0 BS0))"

#define __INV_1 "(and (= ic0 0) (<= 1 BS0))"
#define __INV_2 "false"
#define __INV_3 "(or (and (<= 0 ic0) (<= ic0 1) (<= 2 BS0)) (and (= ic0 1) (<= 1 BS0)))"
#define __INV_4 "(and (= 0 ic0) (<= 1 BS0))"
#define __INV_5 "false"
#define __INV_6 "(or (and (<= 0 ic0) (<= ic0 1) (<= 2 BS0)) (and (= ic0 1) (<= 1 BS0)))"
#define __INV_7 "(and (= 0 ic0) (<= 1 BS0))"
#define __INV_8 "false"
#define __INV_9 "(and (<= 0 ic0) (<= 1 ic0) (<= 1 BS0))"

std::vector<automaton> get_lhs_automaton_concrete(msat_env env) {
  automaton res;
  /* alphabet */
  res.alphabet.insert("tau");
  res.alphabet.insert("wakeup_producer");
  res.alphabet.insert("wakeup_consumer");
  res.alphabet.insert("sleeping_producer");
  res.alphabet.insert("sleeping_consumer");
  res.alphabet.insert("deadlock");
  /* variables */
  res.vars.insert("ic");
  res.vars.insert("BS");
  create_msat_variables(env,res.vars,msat_get_integer_type(env));
  /* init states */
  res.init_states.insert(0);
  /* final states */
  res.final_states.insert(1);
  res.final_states.insert(2);
  res.final_states.insert(3);
  res.final_states.insert(4);
  res.final_states.insert(5);
  res.final_states.insert(6);
  res.final_states.insert(7);
  res.final_states.insert(8);
  res.final_states.insert(9);
  /* create rules */
  create_rule(res.trans,env,0,"tau",1,"(and (= 0 ic1) (< 1 BS1))");

  create_rule(res.trans,env,1,"tau",2,__P12);
  create_rule(res.trans,env,1,"tau",3,__P13);
  create_rule(res.trans,env,3,"tau",1,__P31_2);
  create_rule(res.trans,env,2,"sleeping_producer",2,__P22);

  create_rule(res.trans,env,4,"tau",5,__P12);
  create_rule(res.trans,env,4,"tau",6,__P13);
  create_rule(res.trans,env,6,"tau",4,__P31_2);
  create_rule(res.trans,env,5,"deadlock",5,__P22);

  create_rule(res.trans,env,7,"tau",8,__P12);
  create_rule(res.trans,env,7,"tau",9,__P13);
  create_rule(res.trans,env,9,"tau",7,__P31_2);
  create_rule(res.trans,env,8,"sleeping_producer",8,__P22);

  create_rule(res.trans,env,1,"tau",4,__C12);
  create_rule(res.trans,env,1,"tau",7,__C13);
  create_rule(res.trans,env,7,"tau",1,__C31_2);
  create_rule(res.trans,env,4,"sleeping_consumer",4,__C22);

  create_rule(res.trans,env,2,"tau",5,__C12);
  create_rule(res.trans,env,2,"tau",8,__C13);
  create_rule(res.trans,env,8,"tau",2,__C31_2);

  create_rule(res.trans,env,3,"tau",6,__C12);
  create_rule(res.trans,env,3,"tau",9,__C13);
  create_rule(res.trans,env,9,"tau",3,__C31_2);
  create_rule(res.trans,env,6,"sleeping_consumer",6,__C22);

  create_rule(res.trans,env,8,"wakeup_producer",3,__P23_1__C31);
  create_rule(res.trans,env,6,"wakeup_consumer",7,__P31_1__C23);

  /* invariants */
  add_invariant(res,env,1,__INV_1);
  add_invariant(res,env,2,__INV_2);
  add_invariant(res,env,3,__INV_3);
  add_invariant(res,env,4,__INV_4);
  add_invariant(res,env,5,__INV_5);
  add_invariant(res,env,6,__INV_6);
  add_invariant(res,env,7,__INV_7);
  add_invariant(res,env,8,__INV_8);
  add_invariant(res,env,9,__INV_9);

  std::vector<automaton> res_v;		
  res_v.push_back(res);
  return res_v;
}

automaton get_rhs_automaton_concrete(msat_env env) {
  automaton res;
  /* alphabet */
  res.alphabet.insert("tau");
  res.alphabet.insert("wakeup_producer");
  res.alphabet.insert("wakeup_consumer");
  res.alphabet.insert("sleeping_producer");
  res.alphabet.insert("sleeping_consumer");
  res.alphabet.insert("deadlock");
  /* variables */
  res.vars.insert("ic");
  res.vars.insert("BS");
  create_msat_variables(env,res.vars,msat_get_integer_type(env));
  /* init states */
  res.init_states.insert(0);
  /* final states */
  res.final_states.insert(1);
  /* create rules */
  create_rule(res.trans,env,0,"tau",1,"(and (< ic1 BS1))");

  create_rule(res.trans,env,1,"tau",1,__SAFETY);
  create_rule(res.trans,env,1,"wakeup_producer",1,__SAFETY);
  create_rule(res.trans,env,1,"wakeup_consumer",1,__SAFETY);
  create_rule(res.trans,env,1,"sleeping_producer",1,__SAFETY);
  create_rule(res.trans,env,1,"sleeping_consumer",1,__SAFETY);

  return res;
}
