
#include "mathsat.h"
#include <assert.h>
#include <stdio.h>
#include <map>
#include <string>
#include <stdlib.h>
#include "main.h"

const char * apply_map(char *from,std::map<std::string,std::string> var_map) {
	if (var_map.find(from)==var_map.end()) {
		return from;
	}
	return var_map[from].c_str();
}

/* create a new term with renamed leaves according to the term "orig"  and variable mapping  var_map */
msat_term term_rename(msat_env env, msat_term orig, std::map<std::string,std::string> var_map ){
	msat_term aux, aux2;
	msat_decl decl;
	const char *var_name;
	//std::cout << msat_to_smtlib2(env,orig);
	if (msat_term_is_equal(env,orig)) {
		assert(msat_term_arity(orig)==2);
		aux=term_rename(env,msat_term_get_arg(orig,0),var_map);
		aux2=term_rename(env,msat_term_get_arg(orig,1),var_map);
		return msat_make_equal(env,aux,aux2);
	}
	if (msat_term_is_constant(env,orig)) {
		assert(msat_term_arity(orig)==0);
		var_name=apply_map(msat_decl_get_name(msat_term_get_decl (orig)),var_map);
		decl=msat_find_decl(env,var_name);
		assert(!MSAT_ERROR_DECL(decl));
		return msat_make_constant(env,decl);
	}
	if (msat_term_is_and(env,orig)) {
		assert(msat_term_arity(orig)==2);
		aux=term_rename(env,msat_term_get_arg(orig,0),var_map);
		aux2=term_rename(env,msat_term_get_arg(orig,1),var_map);
		return msat_make_and(env,aux,aux2);
	}
	if (msat_term_is_or(env,orig)) {
		assert(msat_term_arity(orig)==2);
		aux=term_rename(env,msat_term_get_arg(orig,0),var_map);
		aux2=term_rename(env,msat_term_get_arg(orig,1),var_map);
		return msat_make_or(env,aux,aux2);
	}
	if (msat_term_is_not(env,orig)) {
		assert(msat_term_arity(orig)==1);
		aux=term_rename(env,msat_term_get_arg(orig,0),var_map);
		return msat_make_not(env,aux);
	}
	if (msat_term_is_leq(env,orig)) {
		assert(msat_term_arity(orig)==2);
		aux=term_rename(env,msat_term_get_arg(orig,0),var_map);
		aux2=term_rename(env,msat_term_get_arg(orig,1),var_map);
		return msat_make_leq(env,aux,aux2);
	}
	if (msat_term_is_plus(env,orig)) {
		assert(msat_term_arity(orig)==2);
		aux=term_rename(env,msat_term_get_arg(orig,0),var_map);
		aux2=term_rename(env,msat_term_get_arg(orig,1),var_map);
		return msat_make_plus(env,aux,aux2);
	}
	if (msat_term_is_times(env,orig)) {
		assert(msat_term_arity(orig)==2);
		aux=term_rename(env,msat_term_get_arg(orig,0),var_map);
		aux2=term_rename(env,msat_term_get_arg(orig,1),var_map);
		return msat_make_times(env,aux,aux2);
	}

	if (msat_term_is_number(env,orig)) {
		assert(msat_term_arity(orig)==0);
		return orig;
	}
	if (msat_term_is_true(env,orig)) {
		assert(msat_term_arity(orig)==0);
		return orig;
	}
	if (msat_term_is_false(env,orig)) {
		assert(msat_term_arity(orig)==0);
		return orig;
	}
/*	if (msat_term_is_uf(env,orig)){
		printf("uf\n");
		return orig;
	}*/
	printf("rename.c: term_rename: Not implemented !!!\n");
	exit(1);

}

/* compute the number of nodes in the tree reprezentation of the term */
int term_size(msat_env env, msat_term orig ){
	int aux, aux2;
	if (msat_term_is_equal(env,orig)) {
		assert(msat_term_arity(orig)==2);
		aux=term_size(env,msat_term_get_arg(orig,0));
		aux2=term_size(env,msat_term_get_arg(orig,1));
		return aux+aux2+1;
	}
	if (msat_term_is_constant(env,orig)) {
		assert(msat_term_arity(orig)==0);
		return 1;
	}
	if (msat_term_is_and(env,orig)) {
		assert(msat_term_arity(orig)==2);
		aux=term_size(env,msat_term_get_arg(orig,0));
		aux2=term_size(env,msat_term_get_arg(orig,1));
		return aux+aux2+1;
	}
	if (msat_term_is_or(env,orig)) {
		assert(msat_term_arity(orig)==2);
		aux=term_size(env,msat_term_get_arg(orig,0));
		aux2=term_size(env,msat_term_get_arg(orig,1));
		return aux+aux2+1;
	}
	if (msat_term_is_not(env,orig)) {
		assert(msat_term_arity(orig)==1);
		aux=term_size(env,msat_term_get_arg(orig,0));
		return aux+1;
	}
	if (msat_term_is_leq(env,orig)) {
		assert(msat_term_arity(orig)==2);
		aux=term_size(env,msat_term_get_arg(orig,0));
		aux2=term_size(env,msat_term_get_arg(orig,1));
		return aux+aux2+1;
	}
	if (msat_term_is_plus(env,orig)) {
		assert(msat_term_arity(orig)==2);
		aux=term_size(env,msat_term_get_arg(orig,0));
		aux2=term_size(env,msat_term_get_arg(orig,1));
		return aux+aux2+1;
	}
	if (msat_term_is_times(env,orig)) {
		assert(msat_term_arity(orig)==2);
		aux=term_size(env,msat_term_get_arg(orig,0));
		aux2=term_size(env,msat_term_get_arg(orig,1));
		return aux+aux2+1;
	}

	if (msat_term_is_number(env,orig)) {
		assert(msat_term_arity(orig)==0);
		return 1;
	}
	if (msat_term_is_true(env,orig)) {
		assert(msat_term_arity(orig)==0);
		return 1;
	}
	if (msat_term_is_false(env,orig)) {
		assert(msat_term_arity(orig)==0);
		return 1;
	}
/*	if (msat_term_is_uf(env,orig)){
		printf("uf\n");
		return orig;
	}*/
	printf("rename.c: term_size: Not implemented !!!\n");
	exit(1);

}
