#include "main.h"
#include "create_automata.h"

int N=2;

std::string init_symbol = "i"; 

std::string v = "v", v0 = v + "0", v1 = v + "1";
std::string x = "x", x0 = x + "0", x1 = x + "1";
std::string D = "D", D0 = D + "0", D1 = D + "1";

std::vector<automaton> get_lhs_automaton_concrete(msat_env env) {
  std::vector<automaton> res_v;

  for (int i = 1; i <= N; i ++) {
    automaton p;

    std::string alpha_symbol = "a" + std::to_string(i);

    p.alphabet.insert(init_symbol);
    p.alphabet.insert(alpha_symbol);

    p.vars.insert(x);
    p.vars.insert(v);
    p.vars.insert(D);

    create_msat_variables(env,p.vars,msat_get_integer_type(env));

    int s0 = 2*i-2, s1 = 2*i - 1;

    p.init_states.insert(s0);
    p.final_states.insert(s1);

    create_rule(p.trans,env,s0,init_symbol,s1,"(and (= " + x1 + " " + " 0) (= " + v1 + " 1)) (< 0 " + D1 + "))");
    create_rule(p.trans,env,s1,alpha_symbol,s1,"(and (>= " + x0 + " " +  "(* " + std::to_string(i-1) + " " + D0 + ")) (< " + x0 + " " + "(* " + std::to_string(i) + " " + D0 + ")) (= " + x1 + " (+ 1 " + x0 + ")) (= " + v1 + " " + std::to_string(i) + ") (= " + D0 + " " + D1 + "))");

    res_v.push_back(p);
  }

  return res_v;
}

automaton get_rhs_automaton_concrete(msat_env env) {
  automaton m; 

  m.alphabet.insert(init_symbol);

  m.vars.insert(v);
  m.vars.insert(x);
  
  create_msat_variables(env,m.vars,msat_get_integer_type(env));

  m.init_states.insert(0);

  create_rule(m.trans,env,0,init_symbol,1,"(= " + v1 + " 1)");

  std::string alpha_symb = "a1";
  
  m.alphabet.insert(alpha_symb);

  for (int i = 1; i <= N; i ++) {
    std::string alpha_next = "a" + std::to_string(i+1);

    m.alphabet.insert(alpha_next);

    m.final_states.insert(i);

    create_rule(m.trans,env,i,alpha_next,i+1,"(= " + v1 + "(+ " + v0 + " 1))");

    for (int j = 1; j <= i; j ++)
      create_rule(m.trans,env,j,alpha_symb,j,"(= " + v1 + " " + v0 + ")");

    alpha_symb = alpha_next;
  }

  return m;
}
