/* Adam Rogalewicz
 * INCLUDER project
 * distributed under GNU GPL v3 */

#include "main.h"


int final_state(int state, automaton aut, int correction) {
	//the state is state+correction
	if (aut.final_states.find(state+correction)==aut.final_states.end()) return 0;
	else return 1;
}

std::set<pre_post_item> aut_post(int state,  automaton aut) {
	std::set<pre_post_item> result;
	a_lhs lhs_lab;
	lhs_lab.src=state;
	a_rhs rhs_lab;
	for(std::set<std::string>::iterator j=aut.alphabet.begin(); j!=aut.alphabet.end(); j++) {
		lhs_lab.lab=*j;
		for(std::vector<a_rhs>::iterator i=aut.trans[lhs_lab].begin();i!=aut.trans[lhs_lab].end(); ++i) {
			rhs_lab=*i;
			pre_post_item tmp;
			tmp.state=rhs_lab.dest;
			tmp.form=rhs_lab.formula;
			tmp.symbol=lhs_lab.lab;
			result.insert(tmp);
		}
	}
	return result;

}

std::set<pre_post_item> aut_post_correction(int state,  automaton aut, int correction) {
	// the state is state+correction
	// the result is corrected in opposite direction
	std::set<pre_post_item> tmp_res, result;
	int b_corr;
	tmp_res=aut_post(state+correction,aut);
	b_corr=-correction;
	// apply backward correction b_corr
	for(std::set<pre_post_item>::iterator i=tmp_res.begin();i!=tmp_res.end();++i) {
		pre_post_item tmp=*i;
		tmp.state=tmp.state+b_corr;
		result.insert(tmp);
	}
	return result;
}


std::set<pre_post_item> aut_pre(int dest,  automaton aut) {
	std::set<pre_post_item> result;
	a_lhs aux_lhs;
	a_rhs aux2_rhs;
	std::vector<a_rhs> aux_rhs;
	for(std::map<a_lhs,std::vector<a_rhs>>::iterator it=aut.trans.begin(); it!=aut.trans.end();++it) {
		aux_lhs=it->first;
		aux_rhs=it->second;
		for(std::vector<a_rhs>::iterator it2=aux_rhs.begin(); it2!=aux_rhs.end();++it2) {
			aux2_rhs=*it2;
			if (aux2_rhs.dest==dest) {
				pre_post_item tmp;
				tmp.state=aux_lhs.src;
				tmp.symbol=aux_lhs.lab;
				tmp.form=aux2_rhs.formula;
				result.insert(tmp);
			}
		}
	}
	return result;

}

std::set<pre_post_item> aut_pre_correction(int state,  automaton aut, int correction) {
	// the state is state+correction
	// the result is corrected in opposite direction
	std::set<pre_post_item> tmp_res, result;
	int b_corr;
	tmp_res=aut_pre(state+correction,aut);
	b_corr=-correction;
	// apply backward correction b_corr
	for(std::set<pre_post_item>::iterator i=tmp_res.begin();i!=tmp_res.end();++i) {
		pre_post_item tmp=*i;
		tmp.state=tmp.state+b_corr;
		result.insert(tmp);
	}
	return result;
}



msat_term *var_to_q_out(msat_env env, int num, std::set<std::string> vars) {
/* "x.num" is placed to the list "to quantify out" for each variable x from vars */
	msat_term *list;
	list=new msat_term[vars.size()];
	std::string seq,vname;
	char seqtmp[50];
	int q=0;
	snprintf(seqtmp,50,"%i", num); seq=seqtmp;
	for(std::set<std::string>::iterator i=vars.begin();i!=vars.end();++i) {
		vname=*i+seq;
		list[q]=msat_make_constant(env, msat_find_decl(env,vname.c_str()));
		q++;
	}
	return list;
}

msat_term shift_vars_01(msat_env env, msat_term form, std::set<std::string> vars) {
/* each variable x0, is renamed to x1 */
	std::map<std::string,std::string> var_map;
	for(std::set<std::string>::iterator i=vars.begin();i!=vars.end();++i) {
		var_map[*i+"0"]=*i+"1";
		// the commented out next line hopefully does not cause problems
		//AddMissingVar(env,*i + "1",msat_get_integer_type(env));
	}
	return term_rename(env,form,var_map);
}

msat_term presim(std::string symb, int q, int r, int q1, std::vector<std::vector<msat_term>> R, msat_env env,int correction,  automaton aut) {
	// first compute pre_symb(q1,r,R)
	std::set<pre_post_item> r_post=aut_post_correction(r,  aut, correction);
	msat_term pre=msat_make_false(env);
	for(std::set<pre_post_item>::iterator i=r_post.begin();i!=r_post.end();i++) {
		pre_post_item tmp=*i;
		if (tmp.symbol==symb) {
			msat_term tmp_sim=shift_vars_01(env,R[q1][tmp.state], aut.vars);
			pre=msat_make_or(env,pre,msat_make_and(env, tmp_sim, tmp.form));
		}
	}
	//find the formula labeling the transition q -- symb --> q1
	std::set<pre_post_item> q_post=aut_post_correction(q,  aut, correction);
	msat_term qq1_trans=msat_make_false(env);
	for(std::set<pre_post_item>::iterator i=q_post.begin();i!=q_post.end();i++) {
		pre_post_item tmp=*i;
		if ((tmp.symbol==symb)&&(tmp.state==q1)) {
			qq1_trans=tmp.form;
			break;
		}
	}
	// result == \forall x' qq1_trans(x,x') -> pre(x,x')
	msat_term res=msat_make_and(env, qq1_trans, msat_make_not(env, pre));
	msat_term *qvars=var_to_q_out(env, 1, aut.vars);
	res=msat_exist_elim_bypass(env,res,qvars, aut.vars.size() ,MSAT_EXIST_ELIM_ALLSMT_FM);
	return msat_make_not(env,res);
}


void print_sim_ll(std::vector<std::vector<msat_term>> R, int cor, int rhs) {
	std::cout << "============================================\n";
	if (rhs<0) rhs=R.size();
	for(int i=0;i<R.size();i++) {
		assert( R.size()==R[i].size());
		if (i<rhs) std::cout<< i+cor;
		else std::cout<< "R"<< i-rhs;
		for(int j=0;j<R[i].size();j++) {
			if (j<rhs)
				std::cout << " | "<<j+cor<<":"<<msat_term_repr(R[i][j]);
			else
				std::cout << " | R"<<j-rhs<<":"<<msat_term_repr(R[i][j]);
		}
		std::cout << "\n============================================\n";
	}

}
void print_sim(sim_item R) {
	// R.rhs is unused now. Reserved for Xsimulations. Expected value is R.rhs==-1
	assert(R.rhs==-1);
	print_sim_ll(R.sim,R.cor,R.rhs);
}

int find_i(msat_env env, std::vector<std::vector<msat_term>> R,	std::vector<std::vector<msat_term>> Rprev) {
/* find i such that vectors R[i] and Rprev[i] are non-equal
 * return -1 is R==Rprev */
	int i,j;
	for (i=0;i<R.size();i++) {
		for (j=0;j<R[i].size();j++) {
			if (!check_term_equivalence(env,R[i][j],Rprev[i][j])) return i;
		}
	}
	return -1;
}

sim_item compute_simulation_ll(msat_env env, automaton aut, int limit,int size_limit){
	// limit: number of iterations
	// size_limit: maximal number of nodes in the formula representation
	// STATES NUMBERING:
	// LHS (aut): We suppose that states in LHS (aut) are no necessary numbered from 0. The "correction" is equal 
	//    to the smallest state and it is used to decrease size of the simulation matrix
	// state i correspond to the following real state:
	//  "i" corresponds to "i+correction" in the "aut" (the LHS automaton)
	clock_t time_begin = clock();
	int states,correction;
	std::vector<std::vector<msat_term>> Sim,prevSim;
	std::vector<std::vector<int>> cnt;
	std::set<std::string> alpha; //alphabet of aut 
	correction=get_min_states_no(aut); // get the number of the smalles state and use it as a correction -- [0,correction-1] is unused
	states=(get_states_no(aut))-correction;
	alpha=aut.alphabet;
	Sim.resize(states);
	prevSim.resize(states);
	cnt.resize(states);
	// initialize old_sim and sim
	for(int i=0;i<states;i++) {
		Sim[i].resize(states);
		prevSim[i].resize(states);
		cnt[i].resize(states);
		for(int j=0;j<states;j++) {
			prevSim[i][j]=msat_make_true(env);
			cnt[i][j]=limit;
		}
	}
	for(int i=0;i<states;i++) {
		for(int j=0;j<states;j++) {
			if (final_state(i,aut,correction) && (! final_state(j,aut,correction))) {
				Sim[i][j]=msat_make_false(env);
			} else {
				Sim[i][j]=msat_make_true(env);
				std::set<pre_post_item> i_post=aut_post_correction(i,  aut, correction);
				//forall a and l: i -- a --> l is a rule of the automaton aut
				// a==(*i_next).symbol, l==(*i_next).state
				for(std::set<pre_post_item>::iterator i_next=i_post.begin();i_next!=i_post.end();i_next++) {
					Sim[i][j]=msat_make_and(env,Sim[i][j],
						presim((*i_next).symbol,i,j,(*i_next).state,prevSim, env,correction,aut));
				}
			}
			

		}
		 
	}

	int cur_state;
	cur_state=find_i(env,Sim,prevSim);
	while (cur_state>=0) {
		std::cout << "Main loop for: "<< cur_state << "\n";
		//forall i,a: i -- a --> cur_state
		// a==(*q).symbol, i==(*q).state
		std::set<pre_post_item> pre_l=aut_pre_correction(cur_state, aut, correction);
		for(std::set<pre_post_item>::iterator q=pre_l.begin();q!=pre_l.end();q++) {
			int i=(*q).state;
			for(int j=0;j<states;j++) {
				Sim[i][j]=msat_make_and(env,Sim[i][j],
					presim((*q).symbol,i,j,cur_state,Sim,env,correction,aut));
			}					
		}
		for(int j=0;j<states;j++) {

			if (!check_term_equivalence(env, prevSim[cur_state][j], Sim[cur_state][j])) {
				prevSim[cur_state][j]=Sim[cur_state][j];
				if (cnt[cur_state][j]) cnt[cur_state][j]--;
				else Sim[cur_state][j]=msat_make_false(env);
			}
		}
		cur_state=find_i(env,Sim,prevSim);
	}
	sim_item res;
	res.sim=Sim;
	res.cor=correction;
	res.time=(clock()-time_begin)/CLOCKS_PER_SEC;
	res.rhs=-1; // no cross simulation allowed
	return res;


}

simulations compute_simulation(msat_env env, product_automaton lhs, automaton rhs,int limit,int size_limit){
	simulations res;
	for(int i=0; i< lhs.components.size();i++) {
		std::cout << "SIM: Computing simulations for LHS component no: " << i <<"\n";
		res.lhs.push_back( compute_simulation_ll(env,lhs.components[i],limit,size_limit));
		print_sim(res.lhs[i]);
	}
	std::cout << "SIM: Computing simulations for RHS\n";
	res.rhs=compute_simulation_ll(env,rhs,limit,size_limit);
	print_sim(res.rhs);
	std::cout << "SIM: DONE\n";
	return res;
}



