#include "main.h"
#include "create_automata.h"


std::vector<automaton> get_lhs_automaton_concrete(msat_env env){
	automaton res;
	std::vector<automaton> res_v;	
	/* alphabet */
	res.alphabet.insert("init");
	res.alphabet.insert("app");
	res.alphabet.insert("down");
	res.alphabet.insert("up");
	res.alphabet.insert("in");
	res.alphabet.insert("out");
	/* variables */
	res.vars.insert("a");
	res.vars.insert("b");
	res.vars.insert("c");
	res.vars.insert("t");
	DWA_VARIABLE_TYPE=msat_get_rational_type(env); // global type redefinition
	create_msat_variables(env,res.vars,DWA_VARIABLE_TYPE);
	/* init states */
	res.init_states.insert(0);
	/* final states */
	res.final_states.insert(1);
	/* create rules */
	create_rule(res.trans,env,0,"init",1,"(and (= 0 a1) (= 0 b1) (= 0 c1))");
	create_rule(res.trans,env,1,"app",2,"(and (<= t0 t1) (= a1 t0) (= b1 t0) (= c1 t0) (<= (- t1 a1) 5) (< (- t1 b1) 2))");
	create_rule(res.trans,env,2,"in",3,"(and (<= t0 t1) (= a1 a0) (= b1 b0) (= c1 c0) (<= (- t1 a1) 5) (<= 2 (- t0 a0)) (< (- t0 a0) 5) )");
	create_rule(res.trans,env,4,"in",5,"(and (<= t0 t1) (= a1 a0) (= b1 b0) (= c1 c0) (<= (- t1 a1) 5) (<= 2 (- t0 a0)) (< (- t0 a0) 5) )");
	create_rule(res.trans,env,2,"down",4,"(and (<= t0 t1) (= a1 a0) (= b1 b0) (= c1 t0) (<= (- t1 a1) 5) (<= 1 (- t0 b0)) (< (- t0 b0) 2) )");
	create_rule(res.trans,env,3,"down",5,"(and (<= t0 t1) (= a1 a0) (= b1 b0) (= c1 t0) (<= (- t1 a1) 5) (<= 1 (- t0 b0)) (< (- t0 b0) 2) )");
	create_rule(res.trans,env,5,"out",6,"(and (<= t0 t1) (= a1 a0) (= b1 t0) (= c1 c0) (<= (- t0 a0) 5) (<= (- t1 b1) 1) )");
	create_rule(res.trans,env,6,"up",1,"(and (<= t0 t1) (= a1 a0) (= b1 b0) (= c1 c0)  (<= (- t0 b0) 1) )");
	res_v.push_back(res);
	return res_v;



}

automaton get_rhs_automaton_concrete(msat_env env){
	automaton res;
	/* alphabet */
	res.alphabet.insert("init");
	res.alphabet.insert("app");
	res.alphabet.insert("down");
	res.alphabet.insert("up");
	res.alphabet.insert("in");
	res.alphabet.insert("out");
	/* variables */
	res.vars.insert("c");
	res.vars.insert("t");
	DWA_VARIABLE_TYPE=msat_get_rational_type(env); // global type redefinition
	create_msat_variables(env,res.vars,DWA_VARIABLE_TYPE);
	/* init states */
	res.init_states.insert(0);
	/* final states */
	res.final_states.insert(1);
	/* create rules */
	create_rule(res.trans,env,0,"init",1,"(= 0 c1)");
	create_rule(res.trans,env,1,"in",1,"(and (<= t0 t1) (= c1 c0) )");
	create_rule(res.trans,env,1,"up",1,"(and (<= t0 t1) (= c1 c0) )");
	create_rule(res.trans,env,1,"app",1,"(and (<= t0 t1) (= c1 t0) )");
	create_rule(res.trans,env,1,"down",2,"(and (<= t0 t1) (= c1 t0)  (<= (- t1 c1) 5))");
	create_rule(res.trans,env,2,"in",2,"(and (<= t0 t1) (= c1 c0)  (<= (- t1 c1) 5))");
	create_rule(res.trans,env,2,"out",2,"(and (<= t0 t1) (= c1 c0)  (<= (- t1 c1) 5))");
	create_rule(res.trans,env,2,"up",1,"(and (<= t0 t1) (= c1 c0) )");

	return res;

}
